/*
 * This file is part of Nepomuk-KDE Project.
 *
 * Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "metaannotationplugin.h"
#include "annotationpluginwrapper.h"
#include "annotationrequest.h"
#include "annotationpluginfactory.h"


class Nepomuk::MetaAnnotationPlugin::Private
{
public:
    AnnotationPluginWrapper* m_wrapper;

    MetaAnnotationPlugin* q;

    void setupWrapper();
    void _k_newAnnotation( Nepomuk::Annotation* );
    void _k_finished();
};


void Nepomuk::MetaAnnotationPlugin::Private::setupWrapper()
{
    if ( !m_wrapper ) {
        m_wrapper = new AnnotationPluginWrapper( q );
        q->connect( m_wrapper, SIGNAL( newAnnotation( Nepomuk::Annotation* ) ), SLOT( _k_newAnnotation( Nepomuk::Annotation* ) ) );
        q->connect( m_wrapper, SIGNAL( finished() ), SLOT( _k_finished() ) );
    }
}


void Nepomuk::MetaAnnotationPlugin::Private::_k_newAnnotation( Nepomuk::Annotation* anno )
{
    q->newMetaAnnotation( anno );
}


void Nepomuk::MetaAnnotationPlugin::Private::_k_finished()
{
    q->metaAnnotationsFinished();
}


Nepomuk::MetaAnnotationPlugin::MetaAnnotationPlugin( QObject* parent )
    : AnnotationPlugin( parent ),
      d( new Private )
{
    d->q = this;
    d->m_wrapper = 0;
}


Nepomuk::MetaAnnotationPlugin::~MetaAnnotationPlugin()
{
    delete d;
}


void Nepomuk::MetaAnnotationPlugin::getPossibleMetaAnnotations( const AnnotationRequest& request )
{
    d->setupWrapper();
    d->m_wrapper->setPlugins( AnnotationPluginFactory::instance()->getPluginsSupportingAnnotationForResource( request.resource().resourceUri() ) );
    d->m_wrapper->getPossibleAnnotations( request );
}


void Nepomuk::MetaAnnotationPlugin::newMetaAnnotation( Annotation* anno )
{
    addNewAnnotation( anno );
}

void Nepomuk::MetaAnnotationPlugin::metaAnnotationsFinished()
{
    emitFinished();
}

#include "metaannotationplugin.moc"
