/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "olenaplugin.h"
#include "olenatextextractionjob.h"
#include "annotationrequest.h"
#include "simplepimoannotation.h"
#include "pimo.h"
#include "nfo.h"
#include "nie.h"

#include <KIconLoader>
#include <KLocale>
#include <KIO/MimetypeJob>
#include <KIO/Job>
#include <KDebug>

#include <Nepomuk/Resource>
#include <Nepomuk/Variant>
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Types/Property>

#include <QtCore/QFile>


Nepomuk::OlenaAnnotationPlugin::OlenaAnnotationPlugin(QObject* parent, const QList<QVariant>&)
    : MetaAnnotationPlugin(parent),
      m_currentMimeTypeJob( 0 ),
      m_currentTextExtractionJob( 0 )
{
    kDebug();
}


Nepomuk::OlenaAnnotationPlugin::~OlenaAnnotationPlugin()
{
}


void Nepomuk::OlenaAnnotationPlugin::doGetPossibleAnnotations( const AnnotationRequest& request )
{
    kDebug() << request.resource().resourceUri() << request.filter() << request.text().left( 5 );

    if ( m_currentMimeTypeJob ) {
        m_currentMimeTypeJob->disconnect( this );
        m_currentMimeTypeJob->kill();
        m_currentMimeTypeJob = 0;
    }

    if ( m_currentTextExtractionJob ) {
        m_currentTextExtractionJob->disconnect( this );
        m_currentTextExtractionJob->cancel();
        m_currentTextExtractionJob = 0;
    }

    // we use KIO::MimetypeJob instead of Strigi values to be more generic and also be able to handle
    // non-analyzed files
    if ( request.resource().resourceUri().isValid() ) {
        QUrl url = request.resource().property( Nepomuk::Vocabulary::NIE::url() ).toResource().resourceUri();
        if ( !url.isValid() )
            url = request.resource().resourceUri();
        KIO::MimetypeJob* job = KIO::mimetype( url, KIO::HideProgressInfo );
        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotMimetypeJobResult( KJob* ) ) );
        m_currentMimeTypeJob = job;
    }
    else {
        // nothing to be done
        emitFinished();
    }
}


void Nepomuk::OlenaAnnotationPlugin::slotMimetypeJobResult( KJob* job )
{
    m_currentMimeTypeJob = 0;

    KIO::MimetypeJob* mtjob = static_cast<KIO::MimetypeJob*>( job );
    if ( mtjob->mimetype().contains("image") ) {
        m_currentTextExtractionJob = OlenaTextExtractionJob::extractText( mtjob->url() );
        connect( m_currentTextExtractionJob, SIGNAL( result( KJob* ) ), this, SLOT( slotTextExtractionDone(KJob*) ) );
        m_currentTextExtractionJob->start();
    }
    else {
        emitFinished();
    }
}


void Nepomuk::OlenaAnnotationPlugin::slotTextExtractionDone( KJob* )
{
    QString text = m_currentTextExtractionJob->text();
    m_currentTextExtractionJob = 0;

    if ( !text.isEmpty() ) {
        AnnotationRequest request;
        request.setText( text );
        getPossibleMetaAnnotations( request );
    }
    else {
        emitFinished();
    }
}

NEPOMUK_EXPORT_ANNOTATION_PLUGIN( Nepomuk::OlenaAnnotationPlugin, "nepomuk_olenaannotationplugin" )

#include "olenaplugin.moc"
