// Copyright (C) 2009 EPITA Research and Development Laboratory (LRDE)
//
// This file is part of Olena.
//
// Olena is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation, version 2 of the License.
//
// Olena is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Olena.  If not, see <http://www.gnu.org/licenses/>.
//
// As a special exception, you may use this file as part of a free
// software project without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to produce
// an executable, this file does not by itself cause the resulting
// executable to be covered by the GNU General Public License.  This
// exception does not however invalidate any other reasons why the
// executable file might be covered by the GNU General Public License.

#ifndef SCRIBO_TEXT_PBM2OBJ_HH
# define SCRIBO_TEXT_PBM2OBJ_HH

/// \file
///
/// Extract text line objects from a binary image.

#include <mln/core/image/image2d.hh>
#include <mln/value/label_16.hh>
#include <mln/core/alias/neighb2d.hh>
#include <mln/util/array.hh>

#include <scribo/primitive/extract/objects.hh>
#include <scribo/filter/objects_small.hh>
#include <scribo/filter/objects_thin.hh>
#include <scribo/primitive/link/with_several_left_links.hh>
#include <scribo/primitive/link/with_several_right_links.hh>
#include <scribo/primitive/group/from_double_link.hh>
#include <scribo/primitive/group/apply.hh>



namespace scribo
{

  namespace text
  {

    using namespace mln;


    labeled_image< image2d<value::label_16> >
    pbm2obj(const image2d<bool>& input);



# ifndef MLN_INCLUDE_ONLY


    labeled_image< image2d<value::label_16> >
    pbm2obj(const image2d<bool>& input)
    {
      trace::entering("scribo::text::pbm2obj");

      /// Extract text.
      value::label_16 nbboxes;
      typedef image2d<value::label_16> L;
      typedef labeled_image<L> lbl_t;
      lbl_t objects;
      objects = scribo::primitive::extract::objects(input, c8(), nbboxes);

      /// Filter non interesting objects.
      objects = filter::objects_small(objects, 4);
      objects = filter::objects_thin(objects, 2);

      /// Link objects.
      object_links<L>
	left_link  = primitive::link::with_several_left_links(objects, 30),
	right_link = primitive::link::with_several_right_links(objects, 30);

      /// Group objects.
      object_groups<L>
	groups     = primitive::group::from_double_link(objects,
							left_link,
							right_link);

      objects = primitive::group::apply(objects, groups);

      trace::exiting("scribo::text::pbm2obj");
      return objects;
    }


# endif // ! MLN_INCLUDE_ONLY

  } // end of namespace scribo::text

} // end of namespace scribo


#endif // ! SCRIBO_TEXT_PBM2OBJ_HH
