/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "annotationsuggester.h"
#include "annotation.h"

#include <KNotification>
#include <KIcon>
#include <KLocale>
#include <KDebug>


namespace {
    enum Actions {
        ActionCreate = 1,
        ActionDiscard = 2
    };
}

Nepomuk::AnnotationSuggester::AnnotationSuggester( Annotation* anno, const Nepomuk::Resource& res )
    : QObject(),
      m_annotation( anno ),
      m_resource( res )
{
    kDebug();
    KNotification* notification = new KNotification( "FileAnnotationSuggestion",
                                                     0,
                                                     KNotification::Persistent );
    notification->setText( i18n("New file '%1': %2", res.resourceUri().path().section('/', -1), m_annotation->comment() ));
    notification->setPixmap( KIcon( "nepomuk" ).pixmap( 32, 32 ) );
    notification->setActions( QStringList()
                              << i18n( "Create Annotation" )
                              << i18n( "Discard" ) );
    connect( notification, SIGNAL( activated( unsigned int ) ),
             this, SLOT( slotNotificationActionActivated( unsigned int ) ) );

    notification->sendEvent();
}


Nepomuk::AnnotationSuggester::~AnnotationSuggester()
{
    kDebug();
    delete m_annotation;
}


void Nepomuk::AnnotationSuggester::slotNotificationActionActivated( unsigned int action )
{
    kDebug() << action;
    if ( action == ActionCreate ) {
        kDebug() << "ActionCreate";
        m_annotation->create( m_resource );
    }

    deleteLater();
}

#include "annotationsuggester.moc"
