/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "fileannotationservice.h"
#include "fileannotator.h"

#include <KUrl>
#include <KLocale>
#include <KPluginFactory>
#include <KDebug>

#include <Nepomuk/ResourceManager>


/**
 * this marco creates a service factory which can then be found by the Qt/KDE
 * plugin system in the Nepomuk server.
 */
NEPOMUK_EXPORT_SERVICE( Nepomuk::FileAnnotationService, "nepomukfileannotationservice")


Nepomuk::FileAnnotationService::FileAnnotationService( QObject* parent, const QList<QVariant>& )
    : Service( parent )
{
    // only so ResourceManager won't open yet another connection to the nepomuk server
    ResourceManager::instance()->setOverrideMainModel( mainModel() );
}


Nepomuk::FileAnnotationService::~FileAnnotationService()
{
}


void Nepomuk::FileAnnotationService::annotateFile( const QString& uri )
{
    kDebug() << uri;
    FileAnnotator* annotator = new FileAnnotator( QUrl::fromEncoded( uri.toAscii() ) );
    annotator->start();
}

#include "fileannotationservice.moc"
