/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "fileannotator.h"
#include "pimo.h"
#include "nie.h"
#include "annotationsuggester.h"

#include "simplepimoannotation.h"
#include "annotationpluginwrapper.h"
#include "annotationplugin.h"
#include "annotationpluginfactory.h"
#include "annotationrequest.h"

#include <QtDBus>

#include <Soprano/Model>
#include <Soprano/StatementIterator>
#include <Soprano/Statement>
#include <Soprano/Node>
#include <Soprano/NodeIterator>
#include <Soprano/QueryResultIterator>

#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Variant>
#include <Nepomuk/Types/Property>
#include <Nepomuk/Types/Class>

#include <KLocale>
#include <KDebug>

class Nepomuk::FileAnnotator::Private
{
public:
    KUrl m_url;

    AnnotationPluginWrapper* m_wrapper;
    AnnotationPluginFactory* m_pluginFactory;
};


Nepomuk::FileAnnotator::FileAnnotator( const KUrl& url )
    : KJob(),
      d( new Private )
{
    d->m_url = url;
    kDebug() << url;

    d->m_pluginFactory = new AnnotationPluginFactory( this );
    d->m_wrapper = new AnnotationPluginWrapper( this );
    connect( d->m_wrapper, SIGNAL( newAnnotation( Nepomuk::Annotation* ) ),
             this, SLOT( slotNewAnnotation( Nepomuk::Annotation* ) ) );
    connect( d->m_wrapper, SIGNAL( finished() ),
             this, SLOT( slotAnnotationPluginsFinished() ) );
}


Nepomuk::FileAnnotator::~FileAnnotator()
{
    kDebug() << d->m_url;
}


void Nepomuk::FileAnnotator::start()
{
    kDebug() << d->m_url;

    //
    // We ignore files that already have manual annotations
    //
    if ( ResourceManager::instance()->mainModel()
         ->executeQuery( QString( "ask where { "
                                  "?r %1 %2 . "
                                  "{ graph ?g { ?r ?p ?o . } "
                                  "OPTIONAL { ?g <http://www.strigi.org/fields#indexGraphFor> ?rr . } "
                                  "FILTER(!BOUND(?rr)) . "
                                  "} UNION { "
                                  "?t %3 ?r . "
                                  "?t ?p ?o . } "
                                  "}" )
                         .arg( Soprano::Node::resourceToN3( Vocabulary::NIE::url() ) )
                         .arg( Soprano::Node::resourceToN3( d->m_url ) )
                         .arg( Soprano::Node::resourceToN3( Vocabulary::PIMO::groundingOccurrence() ) ),
                         Soprano::Query::QueryLanguageSparql ).boolValue() ) {
        kDebug() << "File already has manual annotations. Ignoring.";
        emitResult();
    }
    else {
        // get annotations from plugins
        Resource fileRes( d->m_url );
        d->m_wrapper->setPlugins( AnnotationPluginFactory::instance()->getPluginsSupportingAnnotationForResource( d->m_url ) );

        AnnotationRequest request( fileRes );
        request.setText( fileRes.property( Nepomuk::Vocabulary::NIE::plainTextContent() ).toString() );
        d->m_wrapper->getPossibleAnnotations( request );
    }
}


void Nepomuk::FileAnnotator::slotNewAnnotation( Nepomuk::Annotation* anno )
{
    ( void )new AnnotationSuggester( anno, d->m_url );
}


void Nepomuk::FileAnnotator::slotAnnotationPluginsFinished()
{
    emitResult();
}

#include "fileannotator.moc"
