/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2009 Thomas Riccardi <riccardi.thomas@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#ifndef _KIO_KMETADATA_NEPOMUKTREE_H_
#define _KIO_KMETADATA_NEPOMUKTREE_H_

#include <kio/forwardingslavebase.h>

class QEventLoop;

namespace Nepomuk
{
class NepomukTreeProtocol : public KIO::ForwardingSlaveBase
{
    Q_OBJECT

public:
    NepomukTreeProtocol(const QByteArray& poolSocket, const QByteArray& appSocket);
    virtual ~NepomukTreeProtocol();

    /**
     * List all groups.
     */
    void listDir(const KUrl& url);

    /**
     * Will be forwarded for files.
     */
    void get(const KUrl& url);

    /**
     * Files will be forwarded.
     * Folders will be created as virtual folders.
     */
    void mimetype(const KUrl& url);

    /**
     * Files will be forwarded.
     * Folders will be created as virtual folders.
     */
    void stat(const KUrl& url);

protected:
    /**
     * reimplemented from ForwardingSlaveBase
     */
    bool rewriteUrl(const KUrl& url, KUrl& newURL);

private:
    bool splitUrl(const KUrl& url, QString& group, QString& path, bool signalError = true);
    void listGroup(const QString& group = QString());
    KIO::UDSEntry statResource(const KUrl& uri);

    /**
     * We use the percent encoded groups uri as directory names.
     * But a second encoding seems to be used internally by the KIO,
     * and replace all our '%' by "%25", so we use '~' instead of '%'
     * as delimiter, since it's an authorized character
     *
     */
    static QUrl nameToUri(const QString& name);
    static QString uriToName(const QUrl& uri);

    KIO::UDSEntry createUDSEntryForGroup(const QUrl& groupUri);

    QString checkParentRelationSPARQLAux(const QString& group, const QString& parentGroup);
    QString checkParentRelationSPARQL(const QString& group, const QString& parentGroup);
    QString checkNoParentSPARQL(const QString& group);
    QString checkTypeSPARQLAux(const QString& group);
    QString checkTypeSPARQL(const QString& group);

    QString selectLinkedResourcesSPARQL(const QString& group);
    QString selectSubGroupsSPARQL(const QString& group);
    QString selectRootGroupsSPARQL();

    bool preCheckUri(const QUrl& uri);

    bool parseBool(const QString& str, bool defaultValue);
    bool parseGroupType(const QString& str);
    bool parseSuperGroup(const QString& str);
    bool parseLink(const QString& str);

    void setGroupType(const QUrl& groupType);
    bool isGroupTypeModified();
    QUrl groupType();
    void setSuperGroup(const QUrl& superGroup);
    QUrl superGroup();
    void setSubGroup(const QUrl& subGroup);
    QUrl subGroup();
    void setLink(const QUrl& link);
    QUrl link();

    class Private;
    Private * d;
};
}

#endif
