/*
    Copyright (c) 2009 Charles Wautier <charles.wautier@gmail.com>
    Copyright (c) 2010 Sebastian Trueg <trueg@kde.org>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "konquerortaskplugin.h"

#include "newsitedialog.h"
#include "ui_newsitedialog.h"
#include "nepomukcontext.h"
#include "annotationmenu.h"

#include "pimo.h"
#include "nfo.h"

#include <soprano/nao.h>
#include <soprano/rdf.h>
#include <Soprano/Node>
#include <Soprano/Model>
#include <Soprano/StatementIterator>
#include <Soprano/QueryResultIterator>
#include <Soprano/Error/Error>

#include "nepomukcontext.h"
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Thing>
#include <Nepomuk/Resource>
#include <Nepomuk/Variant>

#include <KActionMenu>
#include <KAction>
#include <KActionCollection>
#include <KLocale>
#include <KIcon>
#include <KMessageBox>
#include <KPluginFactory>
#include <KPluginLoader>
#include <KTemporaryFile>
#include <KDialog>

#include <QMenu>
#include <QUrl>
#include <QLabel>
#include <QtDBus/QDBusMessage>
#include <QtDBus/QDBusConnection>

#include <kmessagebox.h>
#include <khtml_part.h>
#include <dom/html_document.h>
#include <dom/dom_string.h>


Nepomuk::KonquerorTaskPlugin::KonquerorTaskPlugin(QObject* parent, const QVariantList&)
    : KParts::Plugin(parent)
{
    m_action = new KActionMenu( KIcon( "nepomuk" ),
                                i18n( "Annotate web page" ),
                                actionCollection() );
    m_action->setDelayed( false );
    actionCollection()->addAction("nepomuk_annotate", m_action);

    connect(m_action, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)),
            this, SLOT(slotLinkToCurrentTask(Qt::MouseButtons, Qt::KeyboardModifiers)));

    m_annotationMenu = new Nepomuk::AnnotationMenu();
    m_action->setMenu( m_annotationMenu );
    connect( m_annotationMenu, SIGNAL( resourcesAnnotated() ),
             this, SLOT( slotResourceAnnotatedViaMenu() ) );
    connect( m_annotationMenu, SIGNAL( aboutToBeConstructed(Nepomuk::AnnotationMenu*) ),
             this, SLOT( slotAnnotationMenuAboutToBeConstucted() ) );

    m_context = new Nepomuk::ContextServiceInterface(this);
}


Nepomuk::KonquerorTaskPlugin::~KonquerorTaskPlugin()
{
}


void Nepomuk::KonquerorTaskPlugin::slotLinkToCurrentTask(Qt::MouseButtons, Qt::KeyboardModifiers)
{
    if (m_context->currentContext().isValid()) {
        createWebsiteResource().pimoThing().addProperty(Nepomuk::Vocabulary::PIMO::isRelated(), m_context->currentContext());
    }
    else {
        KMessageBox::sorry( 0, i18n( "No current work context set" ) );
    }
}


void Nepomuk::KonquerorTaskPlugin::slotAnnotationMenuAboutToBeConstucted()
{
    kDebug();
    if (parent() && parent()->inherits("KHTMLPart")) {
        KHTMLPart* htmlPart = static_cast<KHTMLPart*>(parent());
        m_annotationMenu->setResource( htmlPart->url() );
    }
}


void Nepomuk::KonquerorTaskPlugin::slotResourceAnnotatedViaMenu()
{
    kDebug();
    // set properties on the page (after the annotation has already been done)
    createWebsiteResource();
}


Nepomuk::Resource Nepomuk::KonquerorTaskPlugin::createWebsiteResource() const
{
    kDebug();
    if (parent() && parent()->inherits("KHTMLPart")) {
        KHTMLPart* htmlPart = static_cast<KHTMLPart*>(parent());
        Nepomuk::Resource nfoResource(htmlPart->url(), Nepomuk::Vocabulary::NFO::Website());
        if ( !nfoResource.exists() ) {

            // Configure the dialog
            NewSiteDialog dlg;
            dlg.setCaption(i18n("Create Relation From Website"));
            dlg.m_url->setText(htmlPart->url().url());
            dlg.m_title->setText(i18nc("@title:window", "Relate Page"));
//             dlg.m_title->setComment(i18nc("%1 is the label of a resource, %2 is its type",
//                                           "<p>to <emphasis>%1</emphasis> (%2)",
//                                           res.genericLabel(),
//                                           Nepomuk::Types::Class( res.resourceType() ).label() ));

            // set a default label
            dlg.m_labelEdit->setText(htmlPart->htmlDocument().title().string().trimmed());
            dlg.m_labelEdit->setFocus(Qt::OtherFocusReason);

            // exec the dialog -> there is no going back now
            dlg.exec();

            nfoResource.pimoThing().setLabel(dlg.m_labelEdit->text());
        }

        return nfoResource;
    }
    else {
        return Resource();
    }
}

K_PLUGIN_FACTORY(KonquerorTaskPluginFactory, registerPlugin<Nepomuk::KonquerorTaskPlugin>();)
K_EXPORT_PLUGIN(KonquerorTaskPluginFactory("khtml_nepomuk_task_linker"))

#include "konquerortaskplugin.moc"
