/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "simpleresourcemodel.h"

#include <QtCore/QUrl>
#include <QtCore/QList>

#include <nepomuk/resource.h>

#include <KDebug>
#include <KUrl>


class Nepomuk::SimpleResourceModel::Private
{
public:
    QList<Nepomuk::Resource> resources;
};


Nepomuk::SimpleResourceModel::SimpleResourceModel( QObject* parent )
    : ResourceModel( parent ),
      d( new Private() )
{
}


Nepomuk::SimpleResourceModel::~SimpleResourceModel()
{
    delete d;
}


QModelIndex Nepomuk::SimpleResourceModel::indexForResource( const QUrl& uri ) const
{
    Q_ASSERT( uri.isValid() );
    // FIXME: performance
    int i = 0;
    QList<Nepomuk::Resource>::const_iterator end = d->resources.constEnd();
    for ( QList<Nepomuk::Resource>::const_iterator it = d->resources.constBegin(); it != end; ++it ) {
        if ( it->resourceUri() == uri ) {
            return index( i, 0 );
        }
        ++i;
    }

    return QModelIndex();
}


QUrl Nepomuk::SimpleResourceModel::resourceForIndex( const QModelIndex& index ) const
{
    if ( index.isValid() && index.row() < d->resources.count() ) {
        return d->resources[index.row()].resourceUri();
    }
    else {
        return QUrl();
    }
}


int Nepomuk::SimpleResourceModel::rowCount( const QModelIndex& parent ) const
{
    if ( parent.isValid() ) {
        return 0;
    }
    else {
        return d->resources.count();
    }
}


QModelIndex Nepomuk::SimpleResourceModel::index( int row, int column, const QModelIndex& parent ) const
{
    if ( row < rowCount( parent ) && column == 0 ) {
        return createIndex( row, column, 0 );
    }
    else {
        return QModelIndex();
    }
}


void Nepomuk::SimpleResourceModel::setResources( const QList<Nepomuk::Resource>& resources )
{
    d->resources = resources;
    reset();
}


void Nepomuk::SimpleResourceModel::addResources( const QList<Nepomuk::Resource>& resources )
{
    beginInsertRows( QModelIndex(), d->resources.count(), d->resources.count() + resources.count() - 1 );
    d->resources << resources;
    endInsertRows();
}


void Nepomuk::SimpleResourceModel::addResource( const Nepomuk::Resource& resource )
{
    addResources( QList<Resource>() << resource );
}


void Nepomuk::SimpleResourceModel::clear()
{
    d->resources.clear();
    reset();
}

#include "simpleresourcemodel.moc"
