/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) version 3, or any
   later version accepted by the membership of KDE e.V. (or its
   successor approved by the membership of KDE e.V.), which shall
   act as a proxy defined in Section 6 of version 3 of the license.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "guiitemresourcemodel.h"
#include "resourceguiitemfactory.h"
#include "abstractresourceguiitem.h"

#include <Nepomuk/Resource>
#include <KDebug>


Q_DECLARE_METATYPE( Nepomuk::AbstractResourceGuiItem* )

class Nepomuk::GuiItemResourceModel::Private
{
public:
    Private( GuiItemResourceModel* parent )
        : q( parent ) {
    }

    // caching the gui items
    ResourceGuiItemFactory factory;
    AbstractResourceGuiItem::DisplayMode displayMode;
    QHash<QString, AbstractResourceGuiItem*> itemCache;

    AbstractResourceGuiItem* itemForResource( const QUrl& url );
    void _k_dataChanged( const QUrl& uri );

private:
    GuiItemResourceModel* q;
};


Nepomuk::AbstractResourceGuiItem* Nepomuk::GuiItemResourceModel::Private::itemForResource( const QUrl& url )
{
    Q_ASSERT( url.isValid() );
    QHash<QString, AbstractResourceGuiItem*>::const_iterator it = itemCache.constFind( url.toString() );
    if ( it != itemCache.constEnd() ) {
        return it.value();
    }
    else {
        AbstractResourceGuiItem* item = factory.guiItemForResource( url );
        connect( item, SIGNAL( dataChanged( const QUrl& ) ),
                 q, SLOT( _k_dataChanged( const QUrl& ) ) );
        itemCache.insert( url.toString(), item );
        return item;
    }
}


void Nepomuk::GuiItemResourceModel::Private::_k_dataChanged( const QUrl& uri )
{
    kDebug() << uri;
    QModelIndex index = q->indexForResource( uri );
    Q_ASSERT( index.isValid() );
    emit q->dataChanged( index, index );
}


Nepomuk::GuiItemResourceModel::GuiItemResourceModel( QObject* parent )
    : ResourceModel( parent ),
      d( new Private(this) )
{
}


Nepomuk::GuiItemResourceModel::~GuiItemResourceModel()
{
    delete d;
}


QVariant Nepomuk::GuiItemResourceModel::data( const QModelIndex& index, int role ) const
{
    QUrl uri = resourceForIndex( index );
    Nepomuk::Resource res( uri );

    if( uri.isValid() ) {
        if ( role == ResourceGuiItemRole )
            return qVariantFromValue( d->itemForResource( uri ) );

        QVariant v = ResourceModel::data( index, role );
        if ( !v.isValid() )
            v = d->itemForResource( uri )->data( uri, role );

        return v;
    }

    return QVariant();
}

#include "guiitemresourcemodel.moc"
