/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "nepomukimageresourceguiitem.h"

#include <kio/previewjob.h>
#include <kfileitem.h>

#include <QtGui/QPainter>


Nepomuk::ImageResourceGuiItem::ImageResourceGuiItem( QObject* parent, const QList<QVariant>& )
    : GenericResourceGuiItem( parent )
{
}


Nepomuk::ImageResourceGuiItem::~ImageResourceGuiItem()
{
}


void Nepomuk::ImageResourceGuiItem::paint( QPainter* painter, DisplayMode mode, const QStyleOption& option, const QUrl& resource )
{
    if ( option.state & QStyle::State_Selected ) {
        painter->fillRect( option.rect, option.palette.highlight() );
    }

    QPixmap pix = getPreview( resource ).scaled( option.rect.size(), Qt::KeepAspectRatio );
    painter->drawPixmap( option.rect.topLeft(), pix );
}


QSize Nepomuk::ImageResourceGuiItem::sizeHint( DisplayMode mode, const QStyleOption& option, const QUrl& resource )
{
    return getPreview( resource ).size();
}


void Nepomuk::ImageResourceGuiItem::slotGotPreview( const KFileItem& item, const QPixmap& pix )
{
    m_previewCache[item.url()] = pix;
}


void Nepomuk::ImageResourceGuiItem::slotFailed( const KFileItem& )
{
}


QPixmap Nepomuk::ImageResourceGuiItem::getPreview( const QUrl& url )
{
    QHash<QUrl, QPixmap>::const_iterator it = m_previewCache.constFind( url );
    if ( it != m_previewCache.constEnd() ) {
        return *it;
    }
    else {
        KIO::PreviewJob* job = KIO::filePreview( KUrl::List() << url,
                                                 100,
                                                 0,
                                                 0,
                                                 0,
                                                 true,
                                                 false);
        job->setIgnoreMaximumSize(true);

        connect( job, SIGNAL(gotPreview(const KFileItem&, const QPixmap&)),
                 this, SLOT(slotGotPreview(const KFileItem&, const QPixmap&)) );

        m_previewCache[url] = QPixmap();
        return QPixmap();
    }
}

#include "nepomukimageresourceguiitem.moc"
