/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "resourcedisplaywidget.h"
#include "abstractresourceguiitem.h"
#include "resourceguiitemfactory.h"

#include <nepomuk/resource.h>

#include <QtGui/QPainter>
#include <QtGui/QPaintEvent>


class Nepomuk::ResourceDisplayWidget::Private
{
public:
    Resource resource;

    ResourceGuiItemFactory factory;
};


Nepomuk::ResourceDisplayWidget::ResourceDisplayWidget( QWidget* parent )
    : QWidget( parent ),
      d( new Private() )
{
    setMouseTracking( true );
}


Nepomuk::ResourceDisplayWidget::ResourceDisplayWidget( const QUrl& resource, QWidget* parent )
    : QWidget( parent ),
      d( new Private() )
{
    setResource( resource );
    setMouseTracking( true );
}


Nepomuk::ResourceDisplayWidget::~ResourceDisplayWidget()
{
}


QUrl Nepomuk::ResourceDisplayWidget::resource() const
{
    return d->resource.resourceUri();
}


void Nepomuk::ResourceDisplayWidget::setResource( const QUrl& uri )
{
    d->resource = Resource( uri );
}


QSize Nepomuk::ResourceDisplayWidget::sizeHint() const
{
    QStyleOption option;
    option.initFrom( this );
    return d->factory.guiItemForType( d->resource.resourceType() )->sizeHint( AbstractResourceGuiItem::DisplayOptimal, option, d->resource.resourceUri() );
}


QSize Nepomuk::ResourceDisplayWidget::minimumSizeHint() const
{
    QStyleOption option;
    option.initFrom( this );
    return d->factory.guiItemForType( d->resource.resourceType() )->minimumSizeHint( AbstractResourceGuiItem::DisplayOptimal, option, d->resource.resourceUri() );
}


void Nepomuk::ResourceDisplayWidget::paintEvent( QPaintEvent* event )
{
    QStyleOption option;
    option.initFrom( this );
    QPainter painter( this );
    d->factory.guiItemForType( d->resource.resourceType() )->paint( &painter, AbstractResourceGuiItem::DisplayOptimal, option, d->resource.resourceUri() );
}


void Nepomuk::ResourceDisplayWidget::mouseMoveEvent( QMouseEvent* event )
{
    QStyleOption option;
    option.initFrom( this );
    if ( d->factory.guiItemForType( d->resource.resourceType() )->handleEvent( event, AbstractResourceGuiItem::DisplayOptimal, option, d->resource.resourceUri() ) ) {
        update();
    }
    else {
        QWidget::mouseMoveEvent( event );
    }
}


void Nepomuk::ResourceDisplayWidget::mousePressEvent( QMouseEvent* event )
{
    QStyleOption option;
    option.initFrom( this );
    if ( d->factory.guiItemForType( d->resource.resourceType() )->handleEvent( event, AbstractResourceGuiItem::DisplayOptimal, option, d->resource.resourceUri() ) ) {
        update();
    }
    else {
        QWidget::mousePressEvent( event );
    }
}


void Nepomuk::ResourceDisplayWidget::mouseReleaseEvent( QMouseEvent* event )
{
    QStyleOption option;
    option.initFrom( this );
    if ( d->factory.guiItemForType( d->resource.resourceType() )->handleEvent( event, AbstractResourceGuiItem::DisplayOptimal, option, d->resource.resourceUri() ) ) {
        update();
    }
    else {
        QWidget::mouseReleaseEvent( event );
    }
}

#include "resourcedisplaywidget.moc"
