/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "resourceformatting.h"
#include "resourceformattingrule.h"
#include "resourceformattingrule_p.h"

#include <nepomuk/resource.h>
#include <nepomuk/variant.h>

#include <QtCore/QRegExp>
#include <QtCore/QVariant>

#include <KDebug>



Nepomuk::ResourceFormattingRule::ResourceFormattingRule()
    : d( new ResourceFormattingRulePrivate() )
{
}


Nepomuk::ResourceFormattingRule::ResourceFormattingRule( const ResourceFormattingRule& other )
{
    d = other.d;
}


Nepomuk::ResourceFormattingRule::~ResourceFormattingRule()
{
}



Nepomuk::ResourceFormattingRule& Nepomuk::ResourceFormattingRule::operator=( const ResourceFormattingRule& other )
{
    d = other.d;
    return *this;
}


bool Nepomuk::ResourceFormattingRule::isValid() const
{
    // each placeholder in the template needs to have at least one assigned property
    if ( !d->formatTemplate.isEmpty() ) {
        int cnt = d->formatTemplate.count( QRegExp( "\\%\\d" ) );
        if ( cnt > 0 && cnt == d->properties.count() ) {
            for ( int i = 0; i < cnt; ++i ) {
                if ( d->properties[i].isEmpty() ) {
                    return false;
                }
            }
            return true;
        }
    }
    return false;
}


Nepomuk::Types::Class Nepomuk::ResourceFormattingRule::type() const
{
    return d->type;
}


QString Nepomuk::ResourceFormattingRule::formatTemplate() const
{
    return d->formatTemplate;
}


QList<QList<Nepomuk::Types::Property> > Nepomuk::ResourceFormattingRule::properties() const
{
    return d->properties;
}


QString Nepomuk::ResourceFormattingRule::formatResource( const Resource& res, ResourceFormattingFlags flags ) const
{
    if ( !res.hasType( d->type.uri() ) ) {
        kDebug() << res.resourceUri() << "has invalid type. Need type" << d->type.uri();
        return QString();
    }

    if ( !isValid() ) {
        kDebug() << "Invalid rule";
        return QString();
    }

    QString s = d->formatTemplate;

    for ( int i = 0; i < d->properties.count(); ++i ) {
        Variant val = res;
        for ( int j = 0; j < d->properties[i].count(); ++j ) {
            val = val.toResource().property( d->properties[i][j].uri() );
            if ( !val.isValid() ) {
                kDebug() << "Could not find value for property" << d->properties[i][j];
                return QString();
            }
        }
        s = s.arg( formatValue( val, flags ) );
    }

    return s;
}
