/*  Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _AUTOPTR_H_
#define _AUTOPTR_H_

    /**
     * \class AutoPtr autoptr.h "autoptr.h"
     *
     * \brief Simple auto pointer class.
     *
     * AutoPtr is a simple implementation of an "auto pointer" which deletes
     * its content when being destroyed. It doesn't support copy or assignment.
     *
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    template< class T >
    class AutoPtr
    {
    public:
        explicit AutoPtr( T* p )
            : m_p( p )
        {
        }

        ~AutoPtr()
        {
            delete m_p;
        }

        T& operator*()
        {
            return m_p;
        }

        const T& operator*() const
        {
            return m_p;
        }

        T* operator->()
        {
            return m_p;
        }

        const T* operator->() const
        {
            return m_p;
        }

        T* data()
        {
            return m_p;
        }

        const T* data() const
        {
            return m_p;
        }

        const T* constData() const
        {
            return m_p;
        }

        bool isNull() const
        {
            return ( m_p == 0 );
        }

        void detach()
        {
            m_p = 0;
        }

    private:
        AutoPtr( const AutoPtr& );

        AutoPtr& operator==( const AutoPtr& );

        T* m_p;
    };

#endif
