/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_MANAGEABLEMAPPINGSTORAGE_H_
#define _SEMCLIP_MANAGEABLEMAPPINGSTORAGE_H_

#include "remotemappingstorage.h"

class QUrl;

namespace SemClip
{

    class MappingElement;

    /**
     * \class ManageableMappingStorage manageablemappingstorage.h "manageablemappingstorage.h"
     *
     * \brief Interface for mapping storages that can be managed.
     *
     * ManageableMappingStorage defines the common management interface for
     * mapping storages. It consists of functions to add and remove mappings
     * from the storage as well as functions relating to storage persistence.
     * Additionally it also offers a static findMapping wrapper function to
     * acquire a mapping element from any kind of RemoteMappingStorage.
     *
     * \sa MappingElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class ManageableMappingStorage
    {
    public:
        virtual ~ManageableMappingStorage()
        {
        }

        virtual bool addMapping( MappingElement* mapping ) = 0;

        virtual bool removeMapping( const QUrl& source, const QUrl& target ) = 0;

        virtual bool load() = 0;

        virtual bool store() const = 0;

    protected:
        static MappingElement* findMapping( RemoteMappingStorage* remote, const QUrl& source, const QUrl& target )
        {
            return remote->findMappingInternal( source, target );
        }
    };

}

#endif
