/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "mappingelement.h"

#include <QtCore/QFile>

SemClip::MappingElement::MappingElement( const QUrl& source, const QUrl& target, bool isNew /*= true*/ )
    : m_source( source ),
      m_target( target ),
      m_fileName( QString() ),
      m_directory( QString() ),
      m_needsInitialization( !isNew )
{
}

SemClip::MappingElement::~MappingElement()
{
}

QUrl SemClip::MappingElement::sourceUri() const
{
    return m_source;
}

QUrl SemClip::MappingElement::targetUri() const
{
    return m_target;
}

QString SemClip::MappingElement::fileName() const
{
    return m_fileName;
}

void SemClip::MappingElement::setFileName( const QString& fileName )
{
    m_fileName = fileName;

    // make sure the file name does not contain any directory
    int index = m_fileName.lastIndexOf( '/' );
    if ( index != -1 ) {
        m_fileName.remove( 0, index + 1 );
    }
}

QString SemClip::MappingElement::directory() const
{
    return m_directory;
}

void SemClip::MappingElement::setDirectory( const QString& directory )
{
    m_directory = directory;

    // make sure the directory ends with a slash
    if ( !m_directory.endsWith( '/' ) ) {
        m_directory.append( '/' );
    }
}

const void* SemClip::MappingElement::content() const
{
    // lazy initialization
    if ( !genericContent() && m_needsInitialization ) {
        loadContentInternal();
    }

    return genericContent();
}

bool SemClip::MappingElement::loadContent()
{
    return loadContentInternal();
}
