/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_QUERYABLEMAPPINGSTORAGE_H_
#define _SEMCLIP_QUERYABLEMAPPINGSTORAGE_H_

class QUrl;

namespace SemClip
{

    class MappingElement;

    /**
     * \class QueryableMappingStorage queryablemappingstorage.h "queryablemappingstorage.h"
     *
     * \brief Interface for mapping storages that can be queried.
     *
     * QueryableMappingStorage defines the common interface for querying any
     * kind of mapping storage. It consists only of a function findMapping
     * which takes the URIs of a source and target ontology, and returns a
     * mapping element capable of translating the supplied source ontology to
     * the target ontology. If no such mapping is available, a null pointer is
     * returned.
     *
     * \sa MappingElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class QueryableMappingStorage
    {
    public:
        virtual ~QueryableMappingStorage()
        {
        }

        virtual const MappingElement* findMapping( const QUrl& source, const QUrl& target ) = 0;
    };

}

#endif
