/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "sparqlgraphtransformer.h"

#include "graphutility.h"

#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>

SemClip::SparqlGraphTransformer::SparqlGraphTransformer()
    : GraphTransformer(),
      m_transformation( 0 )
{
}

SemClip::SparqlGraphTransformer::~SparqlGraphTransformer()
{
}

Soprano::Model* SemClip::SparqlGraphTransformer::execute()
{
    Soprano::Model* result = SemClip::createModel();
    GraphUtility utility( result );

    foreach ( const QString& query, m_transformation->queries ) {
        Soprano::QueryResultIterator qit = m_graph->executeQuery( query, Soprano::Query::QueryLanguageSparql );

        // continue if we don't have the expected graph result
        if ( !qit.isGraph() ) {
            continue;
        }

        // otherwise merge query result into the result model
        Soprano::StatementIterator sit = qit.iterateStatements();
        utility.mergeStatementIterator( sit );
    }

    return result;
}

void SemClip::SparqlGraphTransformer::setSpecificTransformation( const void* transformation )
{
    m_transformation = static_cast< const SparqlMappingElement::Content* >( transformation);
}
