/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_SPARQLGRAPHTRANSFORMER_H_
#define _SEMCLIP_SPARQLGRAPHTRANSFORMER_H_

#include "graphtransformer.h"
#include "sparqlmappingelement.h"

namespace SemClip
{

    /**
     * \class SparqlGraphTransformer sparqlgraphtransformer.h "sparqlgraphtransformer.h"
     *
     * \brief Graph transformer for SparqlMappingElement.
     *
     * SparqlGraphTransformer inherits from GraphTransformer, and is able
     * to apply mappings of the type SparqlMappingElement.
     *
     * \sa GraphTransformer, SparqlMappingElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class SparqlGraphTransformer : public GraphTransformer
    {
    public:
        /**
         * Default constructor.
         */
        SparqlGraphTransformer();

        /**
         * Default destructor.
         */
        virtual ~SparqlGraphTransformer();

        /**
         * Applies the transformation to the model.
         */
        virtual Soprano::Model* execute();

    protected:
       /**
        * Casts the generic transformation pointer to the used content.
        *
        * @param transformation Generic void pointer to the transformation.
        */
        virtual void setSpecificTransformation( const void* transformation );

    private:
        const SparqlMappingElement::Content* m_transformation;
    };

}

#endif
