/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_SPARQLMAPPINGSTORAGE_H_
#define _SEMCLIP_SPARQLMAPPINGSTORAGE_H_

#include "remotemappingstorage.h"

namespace SemClip
{

    /**
     * \class SparqlMappingStorage sparqlmappingstorage.h "sparqlmappingstorage.h"
     *
     * \brief Adapter to a remote mapping storage prototype.
     *
     * SparqlMappingStorage is an adapter to a specific existing remote mapping
     * storage prototype and inherits from RemoteMappingStorage.
     *
     * The remote mapping storage is accessed trough XML-RPC calls, the service
     * prototype is running at "http://seal.ifi.uzh.ch/mappinglib" and offers
     * only a single public method, "RemoteMappingStorage.getMapping". The
     * method takes two parameters, source and target ontology URI, and returns
     * a byte array as result. If the byte array is not empty, it represents a
     * zip file containing the SPARQL queries implementing the mapping (file
     * 'queries', and a map of prefixes to full namespaces (file 'prefixes').
     *
     * \sa RemoteMappingStorage, SparqlMappingElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class SparqlMappingStorage : public RemoteMappingStorage
    {
    public:
        /**
         * Custom and default constructors.
         *
         * @param service An URL to the remote mapping storage service.
         */
        explicit SparqlMappingStorage( const QUrl& service );

        /**
         * \overload
         */
        SparqlMappingStorage();

        /**
         * Default destructor.
         */
        virtual ~SparqlMappingStorage();

        /**
         * Returns the name of the class.
         *
         * @return The class name.
         */
        virtual QString className() const;

        /**
         * Returns the name of the class.
         *
         * @return The class name.
         */
        static QString classNameStatic();

    protected:
        /**
         * Attempts to retrieve a mapping from the remote service.
         *
         * @param source The URI of the ontology to translate from.
         * @param target The URI of the ontology to translate to.
         * @return A mapping element if available, null otherwise.
         */
        virtual MappingElement* retrieveMapping( const QUrl& source, const QUrl& target );

    private:
        static const char s_sealMappingLib[];
        static const char s_getMappingMethod[];
    };

}

#endif
