/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "xmlrpcwrapper.h"

#include <QtCore/QEventLoop>
#include <QtCore/QString>
#include <QtCore/QVariant>

#include <kxmlrpcclient/client.h>

XmlRpcWrapper::XmlRpcWrapper( const QUrl& service, QObject* parent )
    : QObject( parent )
{
    m_loop = new QEventLoop( this );
    m_client = new KXmlRpc::Client( service, this );
    m_client->setUserAgent( "XmlRpcWrapper/1.0" );
    reset();
}

XmlRpcWrapper::~XmlRpcWrapper()
{
}

void XmlRpcWrapper::reset()
{
    m_resultMode = ResultUndefined;
    m_data = QList< QVariant >();
    m_error = QPair< int, QString >();
}

void XmlRpcWrapper::call( const QString& method, const QList< QVariant >& parameters )
{
    reset();
    m_client->call( method, parameters, this, SLOT( gotData( const QList< QVariant >&, const QVariant& ) ),
                    this, SLOT( gotError( int, const QString&, const QVariant& ) ), QVariant() );
    m_loop->exec();
}

bool XmlRpcWrapper::hasData() const
{
    return ( m_resultMode == ResultData );
}

QList< QVariant> XmlRpcWrapper::data() const
{
    return m_data;
}

bool XmlRpcWrapper::hasError() const
{
    return ( m_resultMode == ResultError );
}

QPair< int, QString > XmlRpcWrapper::error() const
{
    return m_error;
}

void XmlRpcWrapper::gotData( const QList< QVariant >& result, const QVariant& id )
{
    m_resultMode = ResultData;
    m_data = result;
    m_loop->quit();
}

void XmlRpcWrapper::gotError( int errorCode, const QString& errorString, const QVariant& id )
{
    m_resultMode = ResultError;
    m_error = QPair< int, QString >( errorCode, errorString);
    m_loop->quit();
}

#include "xmlrpcwrapper.moc"
