/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _XMLRPCWRAPPER_H_
#define _XMLRPCWRAPPER_H_

#include <QtCore/QList>
#include <QtCore/QObject>
#include <QtCore/QPair>

class QEventLoop;
class QString;
class QUrl;
class QVariant;

namespace KXmlRpc
{
    class Client;
}

    /**
     * \class XmlRpcWrapper xmlrpcwrapper.h "xmlrpcwrapper.h"
     *
     * \brief Simple synchronous wrapper around KXmlRpcClient.
     *
     * XmlRpcWrapper is a simple wrapper around KXmlRpcClient, making its
     * functionality available through synchronous communication. Clients
     * dispatch one single call, and afterwards retrieve the result through
     * either the data or error function. Status can be retrieved in between.
     *
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class XmlRpcWrapper : public QObject
    {
        Q_OBJECT

    public:
        explicit XmlRpcWrapper( const QUrl& service, QObject* parent = 0 );

        ~XmlRpcWrapper();

        void call( const QString& method, const QList< QVariant >& parameters );

        bool hasData() const;

        QList< QVariant> data() const;

        bool hasError() const;

        QPair< int, QString > error() const;

    private slots:
        void gotData( const QList< QVariant >& result, const QVariant& id );

        void gotError( int errorCode, const QString& errorString, const QVariant& id );

    private:
        enum ResultMode {
            ResultUndefined,
            ResultData,
            ResultError
        };

        void reset();

        QEventLoop*           m_loop;
        KXmlRpc::Client*      m_client;
        ResultMode            m_resultMode;
        QList< QVariant >     m_data;
        QPair< int, QString > m_error;
    };

#endif
