/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "ontologyelement.h"

#include <Soprano/Model>

SemClip::OntologyElement::OntologyElement( const QUrl& uri, bool isNew /*= true*/ )
    : m_uri( uri ),
      m_content( 0 ),
      m_needsInitialization( !isNew )
{
}

SemClip::OntologyElement::~OntologyElement()
{
    delete m_content;
}

QUrl SemClip::OntologyElement::uri() const
{
    return m_uri;
}

const Soprano::Model* SemClip::OntologyElement::content() const
{
    // lazy initialization
    if ( !m_content && m_needsInitialization ) {
        loadContentInternal();
    }

    return m_content;
}

void SemClip::OntologyElement::setContent( Soprano::Model* content )
{
    m_needsInitialization = false;
    if ( content != m_content ) {
        delete m_content;
        m_content = content;
    }
}

bool SemClip::OntologyElement::loadContent()
{
    return loadContentInternal();
}
