/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_ONTOLOGYSTORAGE_H_
#define _SEMCLIP_ONTOLOGYSTORAGE_H_

class QString;
class QUrl;

namespace Soprano
{
    class Model;
}

namespace SemClip
{
    class OntologyElement;

    /**
     * \class OntologyStorage ontologystorage.h "ontologystorage.h"
     *
     * \brief Abstract base class for an ontology storage.
     *
     * OntologyStorage defines the common interface for an ontology storage
     * which manages ontology elements and offers functions to retrieve, add
     * and remove ontologies as well as for managing its own persistence.
     *
     * \sa OntologyElement
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class OntologyStorage
    {
    public:
        /**
         * Modes of ontology removal for the ontology storage. Specifies what
         * should be done with the persistent ontology content on removal.
         */
        enum RemovalMode {
            RemoveOnly,      /**< Keep the persistent content. */
            RemoveAndErase   /**< Erase the persistent content as well. */
        };

        virtual ~OntologyStorage()
        {
        }

        virtual const OntologyElement* findOntology( const QUrl& uri ) = 0;

        virtual bool addOntology( const QUrl& uri, Soprano::Model* model ) = 0;

        virtual bool addOntology( const QUrl& uri, const QString& serialization ) = 0;

        virtual bool removeOntology( const QUrl& uri, RemovalMode mode ) = 0;

        virtual bool load() = 0;

        virtual bool store() const = 0;
    };

}

#endif
