/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#include <nepomuk/resource.h>
#include <annotation.h>
#include "pimo.h"
#include "document.h"

using namespace Nepomuk;

class Nepomuk::DocumentPrivate
{
    public:
        DocumentPrivate(Types::Class& c) :
            cls(c), pimo(Types::Class(Nepomuk::Vocabulary::PIMO::Thing())), annotations(QList<
                    Annotation*> ()), label(QString()), comment(QString())
        {
        }
        Types::Class cls;
        Types::Class pimo;
        QList<Nepomuk::Annotation*> annotations;
        QString label;
        QString comment;
        QString extension;
};

Document::Document(Types::Class& cls, QObject* parent) :
    QObject(parent), d(new DocumentPrivate(cls))
{
}

Document::~Document()
{
    delete d;
}

Types::Class Document::typeClass() const
{
    return d->cls;
}

void Document::setTypeClass(const Types::Class& cls)
{
    d->cls = cls;
}

void Document::setPIMOType(const Nepomuk::Types::Class& cls)
{
    d->pimo = cls;
}

Nepomuk::Types::Class Document::PIMOType() const
{
    return d->pimo;
}

void Document::addAnnotation(Annotation* annotation)
{
    addAnnotations(QList<Annotation*>() << annotation);
}

void Document::addAnnotations(QList<Annotation*> annotations)
{
    for (int i = 0; i < annotations.size(); ++i) {
        Annotation* ann = annotations.at(i);
        // important for avoiding deleting things during saving operations
        ann->setParent(this);
        d->annotations.append(ann);
    }
}

QList<Annotation*> Document::annotations() const
{
    return d->annotations;
}

void Document::setLabel(const QString& label)
{
    d->label = label;
}

QString Document::label() const
{
    return d->label;
}

void Document::setComment(const QString& comment)
{
    d->comment = comment;
}

QString Document::comment() const
{
    return d->comment;
}

void Document::print()
{
    qDebug().nospace() << "Label: " << label();
    qDebug().nospace() << "Comment: " << comment();
    qDebug().nospace() << "Type class: " << typeClass().name();
    for (int i = 0; i < annotations().size(); ++i) {
        qDebug().nospace() << "Annotation " << i + 1 << ": " << annotations().at(i)->label();
    }
}

void Document::setFileExtension(const QString& extension)
{
    if (extension.startsWith(".")) {
        d->extension = extension;        
    }
    else {
        d->extension = QString(".").append(extension);
    }
}
            
QString Document::fileExtension() const
{
    return d->extension;
}
