/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) version 3, or any
   later version accepted by the membership of KDE e.V. (or its
   successor approved by the membership of KDE e.V.), which shall
   act as a proxy defined in Section 6 of version 3 of the license.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _NEPOMUK_SMART_FILTER_H_
#define _NEPOMUK_SMART_FILTER_H_

#include <QtCore/QSharedDataPointer>

namespace Soprano {
    class Node;
}

namespace Nepomuk {
    namespace Types {
        class Property;
    }

    class Filter
    {
    public:
        enum Operator {
            None = 0,
            Equals = 1,
            LessThan = 2,
            MoreThan = 4
        };
        Q_DECLARE_FLAGS( Operators, Operator )

        Filter();
        Filter( const Filter& );
        Filter( const Types::Property& property, const Soprano::Node& value, Operators = Equals );
        ~Filter();

        Filter& operator=( const Filter& );

        bool operator==( const Filter& other ) const;

        Types::Property property() const;
        Soprano::Node value() const;
        Operators operators() const;

        /**
         * A tuned version of Nepomuk::Types::Property::label
         */
        QString propertyLabel() const;

        QString toSparqlPattern( const QString& varName ) const;

    private:
        class Private;
        QSharedDataPointer<Private> d;
    };

    uint qHash( const Filter& filter );
}

Q_DECLARE_OPERATORS_FOR_FLAGS(Nepomuk::Filter::Operators)

#endif


