/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#ifndef SMARTANNOTATIONWIDGET_H_
#define SMARTANNOTATIONWIDGET_H_

#include <annotationwidget.h>

namespace Nepomuk
{
    /**
     * @brief Widget for adding personalized annotations through signals and slots.
     * 
     * This subclass of AnnotationWidget extends it by communicating each selected
     * annotation through a signal, instead of creating it directly.
     * This class does not make use of the resource which can be set through the
     * AnnotationWidget::setResource() method.
     * 
     * @sa AnnotationWidget
     * 
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com
     */
    class SmartAnnotationWidget : public AnnotationWidget
    {
        Q_OBJECT
        
        public:
            /**
             * Constructor.
             * 
             * @param parent the parent widget
             */
            SmartAnnotationWidget(QWidget* parent = 0);
            
            /**
             * Destructor
             */
            virtual ~SmartAnnotationWidget();
            
        Q_SIGNALS:
            /**
             * This signals when an annotation is selected and double-clicked
             * by the user.
             * 
             * @param annotation the selected annotation
             */
            void newAnnotation(Nepomuk::Annotation* annotation);
            
        protected:
            /**
             * This method emits the signal newAnnotation() when an annotation
             * is selected and double-clicked.
             * 
             * @param annotation the selected annotation
             */
            virtual void createAnnotation(Nepomuk::Annotation* annotation);
    };

}

#endif // SMARTANNOTATIONWIDGET_H_
