/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
             (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "smartfilemodule.h"
#include "smartfilewidget.h"

#include <kdirselectdialog.h>
#include <kgenericfactory.h>
#include <kpluginfactory.h>
#include <kservice.h>


Nepomuk::SmartFileModule::SmartFileModule( QObject* parent, const QVariantList& )
    : KAbstractFileModule( parent ),
      m_classicFileModule( 0 )
{
}

Nepomuk::SmartFileModule::~SmartFileModule()
{
}

QWidget* Nepomuk::SmartFileModule::createFileWidget(const KUrl& startDir, QWidget *parent)
{
    return new SmartFileWidget( startDir, this, parent );
}

KUrl Nepomuk::SmartFileModule::getStartUrl( const KUrl& startDir, QString& recentDirClass )
{
    // this is only used in native mode -> thus, we can call the classic implementation
    return classicFileModule()->getStartUrl( startDir, recentDirClass );
}

void Nepomuk::SmartFileModule::setStartDir( const KUrl& directory )
{
    // this is only used in native mode -> thus, we can call the classic implementation
    classicFileModule()->setStartDir( directory );
}

KUrl Nepomuk::SmartFileModule::selectDirectory( const KUrl& startDir,
                                                bool localOnly, QWidget *parent,
                                                const QString& caption )
{
    // fall back to the classic implementation for now
    return KDirSelectDialog::selectDirectory(startDir, localOnly, parent, caption);
}

KAbstractFileModule* Nepomuk::SmartFileModule::classicFileModule()
{
    if ( !m_classicFileModule ) {
        if ( KService::Ptr fileModuleService = KService::serviceByDesktopName("kfilemodule") ) {
            m_classicFileModule = fileModuleService->createInstance<KAbstractFileModule>(this);
        }

    }
    Q_ASSERT( m_classicFileModule );
    return m_classicFileModule;
}

K_PLUGIN_FACTORY(KFileModuleFactory, registerPlugin<Nepomuk::SmartFileModule>();)
K_EXPORT_PLUGIN(KFileModuleFactory("nepomuksmartfilemodule"))

#include "smartfilemodule.moc"
