/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
   (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef _SMART_OPEN_WIDGET_H_
#define _SMART_OPEN_WIDGET_H_

#include <QtGui/QWidget>
#include <QtCore/QList>
#include <Soprano/Node>
#include <kabstractfilewidget.h>

#include "smartfilewidget.h"
#include "ui_smartopen.h"

namespace Nepomuk
{
    class SmartFileWidget;

    /**
     * @brief Open widget.
     *
     * This is the open widget class, which provides a UI similar to
     * the standard file dialog one, with important exceptions: no filename
     * field is provided, the filter combobox selects the document class
     * type, and no folders are shown.
     *
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com>
     */
    class SmartOpenWidget: public QWidget, public Ui_SmartOpen
    {
        Q_OBJECT

    public:
        /**
         * Constructor.
         *
         * @param dlg the file widget associated
         * @param parent the file dialog parent (if present)
         */
        SmartOpenWidget(SmartFileWidget* dlg, QWidget* parent);

        /**
         * Destructor.
         */
        virtual ~SmartOpenWidget();

        /**
         * Gets the chosen filename.
         *
         * @return the filename as a string
         */
        QString getFilename() const;

        /**
         * Gets the chosen filename (as a URL).
         *
         * @return the filename as a URL
         */
        KUrl getFilenameUrl() const;

    public Q_SLOTS:
        /**
         * Called when the selection is changed.
         */
        void selectionChanged();

        /**
         * Called when the dialog is closed.
         */
        void dialogClosed();

    private Q_SLOTS:
        void updateForcedFilters();
        void makeConnections(Nepomuk::SmartFileWidget::DialogType type, KAbstractFileWidget::OperationMode mode);

    private:
        class Private;
        Private* d;
    };
}

#endif
