/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>
   (c) 2009 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef _SMART_SAVE_WIDGET_H_
#define _SMART_SAVE_WIDGET_H_

#include <QtGui/QWidget>
#include <QtCore/QList>
#include <kabstractfilewidget.h>
#include <Soprano/Node>

#include "smartfilewidget.h"
#include "ui_smartsave.h"

class KFileFilterCombo;

namespace Nepomuk
{
    class SmartFileWidget;
    class AnnotationPlugin;
    class AnnotationPluginWrapper;

    /**
     * @brief Save widget.
     *
     * This is the save widget provider; its UI differs a lot from the
     * standard file dialog one, providing metadata suggestions based
     * on analysis of the document contents, plus a few input points
     * for adding more specifications of what is being saved.
     *
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com>
     */
    class SmartSaveWidget: public QWidget, public Ui_SmartSave
    {
        Q_OBJECT

    public:
        /**
         * Constructor.
         *
         * @param dlg the file widget associated
         * @param parent the file dialog parent (if present)
         */
        SmartSaveWidget(SmartFileWidget* dlg, QWidget* parent);

        /**
         * Destructor.
         */
        ~SmartSaveWidget();

        /**
         * Gets the chosen filename.
         * 
         * @return the filename as a string
         */
        QString getFilename() const;

        /**
         * Gets the chosen filename (as a URL).
         * 
         * @return the filename as a URL
         */
        KUrl getFilenameUrl() const;

        /**
         * Gets the current instance of the filter widget.
         * 
         * @return the instance of the filter widget
         */
        KFileFilterCombo *filterWidget() const;

    public Q_SLOTS:
        /**
         * Clear any metadata selected by the user.
         */
        void clearTags();
        
        /**
         * Delete any metadata selected by the user.
         */
        void deleteTag();
        
        /**
         * Add a new suggestion.
         */
        void addSuggestion();
        
        /**
         * Add a new recent suggestion.
         */
        void addRecent();
        
        /**
         * Show the PIMO tree.
         */
        void showModelTree();
        
        /**
         * Analyze the label written by the user.
         */
        void analyzeLabel();
        
        /**
         * Analyze the description written by the user.
         */
        void analyzeDescription();

    private:
        void connectSuggestions();
        void launchOverlay(QWidget* widget, AnnotationPlugin* plugin);
        void launchOverlay(QWidget* widget, AnnotationPluginWrapper* wrapper);

        class Private;
        Private* d;

    private Q_SLOTS:
        void addNewClassMenu(const QPoint&);
        void makeConnections(Nepomuk::SmartFileWidget::DialogType type, KAbstractFileWidget::OperationMode mode);
    };
}

#endif
