/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#ifndef SMARTTEXTEDIT_H_
#define SMARTTEXTEDIT_H_

#include <ktextedit.h>
#include <QFocusEvent>

namespace Nepomuk
{
    /**
     * @brief Enhanced version of KTextEdit.
     * 
     * This class offers a version of the KDE text edit,
     * with a new signal emitted when the editing in the
     * text field is finished. It is based on focus lost
     * by this widget.
     * 
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com>
     * 
     * @sa KTextEdit
     */
    class SmartTextEdit : public KTextEdit
    {
        Q_OBJECT
        
        public:
            /**
             * Constructor.
             * 
             * @param parent the parent widget
             */
            SmartTextEdit(QWidget* parent = 0);
            
            /**
             * Constructor with an optional text parameter.
             * 
             * @param text a text to be inserted into the widget
             * @param parent the parent widget
             */
            SmartTextEdit(const QString& text, QWidget* parent = 0);
            
            /**
             * Destructor.
             */
            virtual ~SmartTextEdit();
            
        Q_SIGNALS:
            /**
             * This signal is emitted when the widget loses its
             * focus, thus when the editing of the text is
             * considered to be finished.
             */
            void editingFinished();
            
        protected:
            /**
             * Overridden to emit the editingFnished() signal.
             * 
             * @param event the event of losing the focus
             * 
             * @sa editingFinished()
             */
            void focusOutEvent(QFocusEvent* event);
    };

}

#endif /* SMARTTEXTEDIT_H_ */
