/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#ifndef RECENTANNOTATIONPLUGIN_H_
#define RECENTANNOTATIONPLUGIN_H_

#include <nepomuk/class.h>
#include <annotationplugin.h>
#include <annotationrequest.h>
#include "asyncquery.h"

namespace Nepomuk
{
    class RecentAnnotationPluginPrivate;

    /**
     * @brief Recent annotations plugin.
     *
     * This class provides some recently used annotations in save
     * and load actions.
     *
     * @author Alessandro Sivieri <alessandro.sivieri@gmail.com>
     *
     * @sa AnnotationPlugin
     */
    class RecentAnnotationPlugin: public Nepomuk::AnnotationPlugin
    {
        friend class RecentAnnotationPluginPrivate;

        Q_OBJECT

        public:
            /**
             * Constructor.
             */
            RecentAnnotationPlugin(QObject* parent = 0);

            /**
             * Destructor.
             */
            virtual ~RecentAnnotationPlugin();

            /**
             * Set limit for recent annotations to be produced.
             *
             * @param limit the new limit
             */
            void setLimit(int limit);

            /**
             * Get limit for recent annotations produced.
             *
             * @return the current limit
             */
            int limit();

        protected:
            /**
             * This method provides the recent annotations, through
             * the given signals.
             *
             * @param request used for passing the type of the document
             */
            void doGetPossibleAnnotations(const AnnotationRequest& request);

        private:
            RecentAnnotationPluginPrivate* d;

        private Q_SLOTS:
            void queryNextReady(Soprano::Util::AsyncQuery* query);
            void queryFinished(Soprano::Util::AsyncQuery* query);
    };

}

#endif /* RECENTANNOTATIONPLUGIN_H_ */
