/*
   Copyright (C) 2009 by Charles Wautier <charles.wautier@gmail.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

//FIXME
// Otherwise, QByteArray.append() can't take a QString parameter
#undef QT_NO_CAST_TO_ASCII

#include "kresourcetreeview.h"
#include <kdebug.h>
#include <KUrl>
#include <kio/global.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <Nepomuk/ResourceManager>
#include <QDBusConnection>
#include <QDBusConnectionInterface>
#include <QByteArray>
#include <QHeaderView>
#include <QLabel>
#include <QDBusInterface>

static const char* protocol = "nepomuktree";

KResourceTreeView::KResourceTreeView(QWidget* parent = 0)
        : KFileTreeView::KFileTreeView(parent)
{
    this->hide();
    QDBusConnection connection = QDBusConnection::sessionBus();
    QDBusInterface* remoteApp = new QDBusInterface("org.kde.NepomukServer",
            "/servicemanager",
            "org.kde.nepomuk.ServiceManager",
            connection,
            this);
    QList<QVariant> args;
    args.append("nepomukstorage");
    QDBusReply<bool> reply = remoteApp->callWithArgumentList(QDBus::Block, "isServiceInitialized", args);

    if (reply.isValid() && reply.value()) {
        kDebug() << "nepomukstorage service initialized";
        init();
    } else {
        // FIXME : The following connection could probably be done through remoteApp
        if (connection.connect(QString(),
                               QString(),
                               QString("org.kde.nepomuk.ServiceManager"),
                               QString("serviceInitialized"),
                               this,
                               SLOT(slotInit(const QString &)))) {
            kDebug() << "Waiting for nepomukstorage";
        } else {
            kDebug() << "Failed to connect with QDBus - Undefined behavior";
        }
    }
}
void KResourceTreeView::slotInit(const QString& s)  //,const QString&, const QString & )
{
    if (s == QString("nepomukstorage")) init();
}

void KResourceTreeView::init()
{
    // Hide every column but the names
    header()->hide();
    for (int i = 1 ; i < model()->columnCount() ; ++i) {
        setColumnHidden(i, true);
    }

    // Set the root url with the protocol and default parameters
    KUrl url;
    url.setProtocol(QString(protocol));
    //FIXME : shoudn t be necessary ??
    url.adjustPath(KUrl::AddTrailingSlash);
    setRootUrl(url);

    connect(this, SIGNAL(currentChanged(const KUrl&)),
            this, SLOT(proxy_currentChanged(const KUrl&)));

    this->show();
}
void KResourceTreeView::proxy_currentChanged(const KUrl& url) const
{
    emit currentUriChanged(urlToUri(url));
}

KUrl KResourceTreeView::urlToUri(const KUrl& url) const
{
    if (url.protocol() == protocol) {
        KUrl uri;
        if (KIO::StatJob* job = KIO::stat(url, KIO::HideProgressInfo)) {
            job->setAutoDelete(false);
            if (KIO::NetAccess::synchronousRun(job, 0)) {
                uri = KUrl(job->statResult().stringValue(KIO::UDSEntry::UDS_EXTRA));
            }
            delete job;
        }
        return uri.isEmpty() ? KUrl() : uri;
    } else {
        return KUrl();
    }
}

KUrl KResourceTreeView::selectedUri() const
{
    return urlToUri(selectedUrl());
}

KUrl KResourceTreeView::currentUri() const
{
    return urlToUri(currentUrl());
}

void KResourceTreeView::setRootUrl(const KUrl& url)
{
    if (url.protocol() == protocol) KFileTreeView::setRootUrl(url);
}

void KResourceTreeView::setRootUrl(const KUrl& typeClass, const KUrl& superProperty, const KUrl& linkProperty,
                                   bool typeInheritance, bool superInheritance, bool linkInheritance,
                                   const KUrl& baseUrl)
{
    KUrl url;

    //FIXME : setProtocol only add a ':' so I have to use addPath
    url.setProtocol(protocol);
    url.adjustPath(KUrl::AddTrailingSlash);

    url.addPath(baseUrl.url());
    url.addQueryItem(QString("type"), typeClass.url());
    url.addQueryItem(QString("super"), superProperty.url());
    url.addQueryItem(QString("link"), linkProperty.url());
    url.addQueryItem(QString("typeInheritance"), typeInheritance ? "1" : "0");
    url.addQueryItem(QString("superInheritance"), superInheritance ? "1" : "0");
    url.addQueryItem(QString("linkInheritance"), linkInheritance ? "1" : "0");
    setRootUrl(url);
}

#include "kresourcetreeview.moc"
