/*
   Copyright (C) 2009 by Dupuy Mathieu <mdupuy@mandriva.com>
   Copyright (C) 2009 by Sebastian Trueg <trueg at kde.org>
   Copyright (C) 2009 by Charles Wautier <charles.wautier@gmail.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "newtaskdialog.h"
#include "ui_newtaskdialog.h"
#include "tasktreedialog.h"

#include "nepomukcontext.h"
#include "pimo.h"
#include "tmo.h"
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>
#include <Nepomuk/Thing>
#include <Nepomuk/Variant>

#include <KDebug>

class NewTaskDialog::Private : public Ui::NewTaskDialog
{
public:
    Nepomuk::Resource otherTask;
    Nepomuk::ContextServiceInterface* context;
};

NewTaskDialog::NewTaskDialog(QWidget* parent)
        : KDialog(parent), d(new Private)
{
    setWindowIcon(KIcon("nepomuk"));
    // So that the advanced button work well
    setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    setMaximumHeight(10);
    //setButtons(KDialog::Ok|KDialog::Cancel|KDialog::User1);
    //button(User1)->setText("Advanced >>");
    //button(User1)->setCheckable(true);

    // Dialog conf
    enableButtonOk(false);
    setPlainCaption(i18n("New Task"));

    // Ui conf
    d->setupUi(mainWidget());
    d->m_nameEdit->setFocus();
    d->m_iconButton->setIcon("view-calendar-tasks");
    slotAdvanced(false);

    connect(d->m_advButton, SIGNAL(toggled(bool)),
            this, SLOT(slotAdvanced(bool)));

    connect(d->m_endBox, SIGNAL(stateChanged(int)),
            this, SLOT(slotEnableEndDate(int)));

    connect(d->m_nameEdit, SIGNAL(textChanged(const QString&)),
            this, SLOT(slotLabelChanged(const QString&)));
    connect(d->m_setTaskButton, SIGNAL(released()),
            this, SLOT(slotSetParentTaskClicked()));

    connect(d->m_otherRadio, SIGNAL(clicked()),
            this, SLOT(slotRadioOtherParentTaskToggled()));

    // Set context
    d->context = new Nepomuk::ContextServiceInterface();
    Nepomuk::Resource cc = d->context->currentContext();
    if (cc.isValid()) {
        d->otherTask = cc;
        d->m_otherRadio->setText(d->otherTask.genericLabel());
    } else {
        d->otherTask = Nepomuk::Resource();
        d->m_otherRadio->setText(i18n("Other:"));
    }
}


NewTaskDialog::~NewTaskDialog()
{
    delete d->context;
    delete d;
}

void NewTaskDialog::slotAdvanced(bool b)
{
    d->m_frame->setVisible(b);
    b ? d->m_advButton->setText("<<< " + i18n("Advanced")) : d->m_advButton->setText(i18n("Advanced") + " >>>");
}

void NewTaskDialog::slotLabelChanged(const QString& text)
{
    enableButtonOk(!text.isEmpty());
}

void NewTaskDialog::slotRadioOtherParentTaskToggled()
{
    if (!d->otherTask.isValid()) slotSetParentTaskClicked();
}

void NewTaskDialog::slotSetParentTaskClicked()
{
    Nepomuk::Resource res = TaskTreeDialog::selectOneTask(this);
    if (res.isValid()) {
        d->otherTask = res;
        d->m_otherRadio->setText(res.genericLabel());
        d->m_otherRadio->setChecked(true);
    } else {
        d->m_noneRadio->setChecked(true);
    }
}

void NewTaskDialog::slotEnableEndDate(int e)
{
    d->m_endDate->setEnabled(e);
}

Nepomuk::Resource NewTaskDialog::createNewTask(QWidget* parent)
{
    NewTaskDialog dlg(parent);

    if (dlg.exec() == Accepted) {
        Nepomuk::Thing Task(dlg.d->m_nameEdit->text() , Nepomuk::Vocabulary::TMO::Task());
        writeProperties(dlg, Task);
        return Task;
    } else {
        return Nepomuk::Resource();
    }
}

void NewTaskDialog::edit(Nepomuk::Resource& Task, QWidget* parent)
{
    Q_ASSERT(Task.hasType(Nepomuk::Vocabulary::TMO::Task()) == false);
    NewTaskDialog dlg(parent);

    dlg.setCaption(i18n("Edit task"));
    readProperties(dlg, Task);
    if (dlg.exec() == Accepted) {
        writeProperties(dlg, Task);
    }
}

void NewTaskDialog::writeProperties(const NewTaskDialog& dlg, Nepomuk::Resource& Task)
{
    Task.setLabel(dlg.d->m_nameEdit->text());

    if (!dlg.d->m_descEdit->toPlainText().isEmpty()) {
        Task.setDescription(dlg.d->m_descEdit->toPlainText());
    }

    Task.addSymbol(dlg.d->m_iconButton->icon());

    // Parent task
    if (dlg.d->m_otherRadio->isChecked()) {
        Task.setProperty(Nepomuk::Vocabulary::TMO::superTask(), dlg.d->otherTask);
    }

    // Priority
    if (dlg.d->m_highRadio->isChecked()) {
        Task.setProperty(Nepomuk::Vocabulary::TMO::priority(), Nepomuk::Vocabulary::TMO::TMO_Instance_Priority_High());
    } else if (dlg.d->m_lowRadio->isChecked()) {
        Task.setProperty(Nepomuk::Vocabulary::TMO::priority(), Nepomuk::Vocabulary::TMO::TMO_Instance_Priority_Low());
    } else {
        Task.setProperty(Nepomuk::Vocabulary::TMO::priority(), Nepomuk::Vocabulary::TMO::TMO_Instance_Priority_Medium());
    }

    // Dates
    if (dlg.d->m_endBox->checkState())
        Task.setProperty(Nepomuk::Vocabulary::TMO::dueDate(), QDateTime( dlg.d->m_endDate->date(), QTime( 23, 59, 59 ) ) );
}

void NewTaskDialog::readProperties(NewTaskDialog& dlg, const Nepomuk::Resource& Task)
{
    dlg.d->m_nameEdit->setText(Task.label());
    // Icon
    if (Task.symbols().size() != 0)
        dlg.d->m_iconButton->setIcon(Task.symbols().first());
    // description
    dlg.d->m_descEdit->setText(Task.description());

    // Parent Task
    if (Task.hasProperty(Nepomuk::Vocabulary::TMO::superTask())) {
        dlg.d->otherTask = Task.property(Nepomuk::Vocabulary::TMO::superTask()).toUrl();
        kDebug() << "TOTO ! !" << dlg.d->otherTask;
        dlg.d->m_otherRadio->setText(dlg.d->otherTask.label());
        dlg.d->m_otherRadio->setChecked(true);
    }

    // Priority
    QUrl priority = Task.property(Nepomuk::Vocabulary::TMO::priority()).toUrl();
    if (priority == Nepomuk::Vocabulary::TMO::TMO_Instance_Priority_High()) {
        dlg.d->m_highRadio->setChecked(true) ;
    } else if (priority == Nepomuk::Vocabulary::TMO::TMO_Instance_Priority_Low()) {
        dlg.d->m_lowRadio->setChecked(true);
    } else {
        dlg.d->m_mediumRadio->setChecked(true);
    }

    // Dates
    if (Task.hasProperty(Nepomuk::Vocabulary::TMO::dueDate())) {
        dlg.d->m_endBox->setChecked(true);
        dlg.d->m_endDate->setEnabled(true);
        dlg.d->m_endDate->setDate(Task.property(Nepomuk::Vocabulary::TMO::dueDate()).toDateTime().date());
    }
}

#include "newtaskdialog.moc"
