/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) version 3, or any
   later version accepted by the membership of KDE e.V. (or its
   successor approved by the membership of KDE e.V.), which shall
   act as a proxy defined in Section 6 of version 3 of the license.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _NEPOMUK_RESOURCE_NAVIGATOR_P_H_
#define _NEPOMUK_RESOURCE_NAVIGATOR_P_H_

#include <Nepomuk/Types/Class>
#include <Nepomuk/Types/Property>
#include <Nepomuk/Resource>
#include <Nepomuk/Query/Term>

#include <QtCore/QHash>
#include <QtGui/QPushButton>
#include <QtGui/QToolButton>
#include <QtGui/QPainter>
#include <QtGui/QStyleOption>

#include "resourcenavigator.h"
#include "nepomukcontext.h"

class QFrame;
class QPushButton;
class QToolButton;

namespace Nepomuk {

    class ResourceButton;

    class NavigatorButton : public QToolButton
    {
    public:
        NavigatorButton( QWidget* parent )
            : QToolButton( parent ),
              m_nextButton( 0 ) {
            setAutoRaise( true );
            setArrowType( Qt::RightArrow );
            setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum ) );
            setPopupMode( QToolButton::InstantPopup );
        }
        ~NavigatorButton() {
            delete m_nextButton;
        }

        QSize sizeHint() const {
            int width = height() / 2;
            if (width < 4) {
                width = 4;
            }
            return QSize( width + 4, QToolButton::sizeHint().height() );
        }

        QAbstractButton* m_nextButton;

    protected:
        void paintEvent( QPaintEvent* event ) {
            Q_UNUSED(event);

            QPainter painter(this);

            QStyleOption option;
            option.initFrom(this);
            option.rect = rect();
            option.palette = palette();

            if ( layoutDirection() == Qt::LeftToRight ) {
                style()->drawPrimitive(QStyle::PE_IndicatorArrowRight, &option, &painter, this);
            }
            else {
                style()->drawPrimitive(QStyle::PE_IndicatorArrowLeft, &option, &painter, this);
            }
        }
    };

    class ResourceButton : public QPushButton
    {
    public:
        ResourceButton( QWidget* parent )
            : QPushButton( parent ),
              m_nextButton( 0 ) {
            setFlat( true );
        }
        ~ResourceButton() {
            delete m_nextButton;
        }

        Resource m_resource;
        NavigatorButton* m_nextButton;
    };


    class ResourceNavigatorPrivate
    {
    public:
        ResourceNavigator* q;

        Types::Class m_topLevelType;
        Types::Class m_subLevelType;
        Types::Property m_parentRelationProperty;
        bool m_allowResourceCreation;

        QFrame* m_mainFrame;

        /// The first button which shows the selection of all top level resources
        NavigatorButton* m_topLevelButton;

        Resource m_currentResource;

        ContextServiceInterface* m_contextServiceInterface;

        void reset();
        ResourceButton* createResourceButton( const Nepomuk::Resource& res );
        NavigatorButton* createNavigatorButton( const Nepomuk::Resource& res );
        NavigatorButton* createButtons( const Nepomuk::Resource& res, NavigatorButton* lastButton );

        void _k_resourceActivated( Nepomuk::NavigatorButton*, const Nepomuk::Resource& res );
        void _k_resourceButtonClicked();
    };
}

#endif
