/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) version 3, or any
   later version accepted by the membership of KDE e.V. (or its
   successor approved by the membership of KDE e.V.), which shall
   act as a proxy defined in Section 6 of version 3 of the license.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourcenavigatormenu_p.h"
#include "resourcenavigator_p.h"
#include "newresourcedialog.h"
#include "pimo.h"

#include <Nepomuk/ResourceManager>
#include <Nepomuk/Variant>
#include <Nepomuk/Query/Query>
#include <Nepomuk/Query/ResourceTypeTerm>
#include <Nepomuk/Query/ResourceTerm>
#include <Nepomuk/Query/ComparisonTerm>
#include <Nepomuk/Query/AndTerm>

#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/Node>
#include <Soprano/Vocabulary/RDFS>

#include <KDebug>

Nepomuk::ResourceNavigatorMenu::ResourceNavigatorMenu( ResourceNavigatorPrivate* p, NavigatorButton* parent )
    : QMenu( parent ),
      m_navigator( p ),
      m_button( parent ),
      m_createNewAction( 0 )
{
    connect( this, SIGNAL( aboutToShow() ), SLOT( slotAboutToShow() ) );
    connect( this, SIGNAL( triggered(QAction*) ), SLOT( slotTriggered(QAction*) ) );
}


Nepomuk::ResourceNavigatorMenu::~ResourceNavigatorMenu()
{
}


void Nepomuk::ResourceNavigatorMenu::setResource( const Nepomuk::Resource& res )
{
    m_resource = res;
}


void Nepomuk::ResourceNavigatorMenu::slotAboutToShow()
{
    kDebug();
    clear();

    // query all top-level items and show them in a menu.
    // TODO: if we have more than N results show the first M results (maybe even based on usage)
    // and then a "more" button for the rest
    QString query;
    QString typeName;
    if ( m_resource.isValid() ) {
        query = Query::Query( Query::AndTerm( Query::ResourceTypeTerm( m_navigator->m_subLevelType ),
                                              Query::ComparisonTerm( m_navigator->m_parentRelationProperty,
                                                                     Query::ResourceTerm( m_resource ) ) ) )
                .toSparqlQuery( Query::Query::HandleInverseProperties );
        typeName = m_navigator->m_subLevelType.label();
    }
    else {
        query = Query::Query( Query::ResourceTypeTerm( m_navigator->m_topLevelType ) )
                .toSparqlQuery();
        typeName = m_navigator->m_topLevelType.label();
    }

    Soprano::QueryResultIterator it
        = ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
    while ( it.next() ) {
        Resource project( it["r"].uri() );
        QAction* a = addAction( project.genericLabel() );
        a->setData( project.resourceUri() );
    }

    if ( m_navigator->m_allowResourceCreation ) {
        m_createNewAction = addAction( i18n( "Create new %1...", typeName ) );
    }

    kDebug() << actions().count();
}


void Nepomuk::ResourceNavigatorMenu::slotTriggered( QAction* action )
{
    kDebug();
    if ( action == m_createNewAction ) {
        m_createNewAction = 0;
        Resource res = NewResourceDialog::createResource( m_resource.isValid() ? m_navigator->m_subLevelType : m_navigator->m_topLevelType, this );
        if ( res.isValid() ) {
            if ( m_resource.isValid() )
                m_resource.addProperty( Vocabulary::PIMO::hasPart(), res );
            emit resourceActivated( m_button, res );
        }
    }
    else {
        QUrl uri = action->data().toUrl();
        emit resourceActivated( m_button, uri );
    }
}

#include "resourcenavigatormenu_p.moc"
