/*
 * This file is part of the Nepomuk KDE project.
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

/*
 * This file has been generated by the Nepomuk Resource class generator.
 * DO NOT EDIT THIS FILE.
 * ANY CHANGES WILL BE LOST.
 */

#ifndef _TASK_H_
#define _TASK_H_

class QDateTime;
class QDate;
class QTime;

namespace Nepomuk {
    class Task;
    class Resource;

}

#include <nepomuk/resource.h>


namespace Nepomuk {

    /**
     * The tmo:task is the central entity of the tmo. Task can range 
     * from vague things to be possibly done in e distant future to concrete 
     * things to be done in a precise foreseeable manner. It is not unrealistic 
     * to assume that knowledge worker have hundred or more tasks a 
     * day. A task can have subtasks. Or the other way around, a task 
     * can be assigned to a supertask. A subtask is an independent task, 
     * which refers to another task via a subtask relation. Usually 
     * this means that the execution of the subtask is required in order 
     * to fulfil the execution of the super task. But finally this is 
     * left to the user; the user is free to finish a supertask while 
     * some subtasks are not finished. Subtasks can be used to structure 
     * complex task into manageable parts. Consequently subtasks 
     * can also be used to transfer a certain aspect of a complex task 
     * to other participants. A tmo:Task is modelled as a subclass 
     * of pimo:ProcessThing which is a subclass of pimo:Thing. This 
     * was done because the information artefacts which a knowledge 
     * worker has in his mind are supposed to be represented in the PIMO 
     * and tasks can be such concepts. This implies practically that 
     * tasks are not only visible within special task management applications 
     * but also within PIMO editing and viewing applications . This 
     * does not necessarily mean that in such a generic PIMO editing 
     * environment all task properties can be altered but rather that 
     * the task is at least displayed by its name. 
     */
    class  Task : public Resource
    {
    public:
        /**
         * Create a new empty and invalid Task instance
         */
        Task();
        Task( ResourceManager* );

        /**
         * Default copy constructor
         */
        Task( const Task& );
        Task( const Resource& );

        /**
         * Create a new Task instance representing the resource
         * referenced by \a uriOrIdentifier.
         */
        Task( const QString& uriOrIdentifier );
        Task( const QString& uriOrIdentifier, ResourceManager* );

        /**
         * Create a new Task instance representing the resource
         * referenced by \a uri.
         */
        Task( const QUrl& uri );
        Task( const QUrl& uri, ResourceManager* );
        ~Task();

        Task& operator=( const Task& );

        /**
         * Get property 'dueDate'. The dueDate according to popular time 
         * management paradigms. 
         */
        QDateTime dueDate() const;

        /**
         * Set property 'dueDate'. The dueDate according to popular time 
         * management paradigms. 
         */
        void setDueDate( const QDateTime& value );

        /**
         * \return The URI of the property 'dueDate'. 
         */
        static QUrl dueDateUri();

        /**
         * Get property 'actualCompletion'. The degree (in percent) 
         * to which a task is completed. 
         */
        double actualCompletion() const;

        /**
         * Set property 'actualCompletion'. The degree (in percent) 
         * to which a task is completed. 
         */
        void setActualCompletion( const double& value );

        /**
         * \return The URI of the property 'actualCompletion'. 
         */
        static QUrl actualCompletionUri();

        /**
         * Get property 'urgency'. The urgence of a task according to the 
         * "ABC analysis" (see http://en.wikipedia.org/wiki/Time_management). 
         */
        Resource urgency() const;

        /**
         * Set property 'urgency'. The urgence of a task according to the 
         * "ABC analysis" (see http://en.wikipedia.org/wiki/Time_management). 
         */
        void setUrgency( const Resource& value );

        /**
         * \return The URI of the property 'urgency'. 
         */
        static QUrl urgencyUri();

        /**
         * Get property 'personInvolvement'. connects the complex (n-ary, 
         * role described) PersonInvolvement and the task. See PersonInvolvement 
         * for details. 
         */
        QList<Resource> personInvolvements() const;

        /**
         * Set property 'personInvolvement'. connects the complex (n-ary, 
         * role described) PersonInvolvement and the task. See PersonInvolvement 
         * for details. 
         */
        void setPersonInvolvements( const QList<Resource>& value );

        /**
         * Add a value to property 'personInvolvement'. connects the 
         * complex (n-ary, role described) PersonInvolvement and the 
         * task. See PersonInvolvement for details. 
         */
        void addPersonInvolvement( const Resource& value );

        /**
         * \return The URI of the property 'personInvolvement'. 
         */
        static QUrl personInvolvementUri();

        /**
         * Get property 'priority'. The priority of a task. See http://en.wikipedia.org/wiki/Time_management 
         * for different approaches on finding priorities. 
         */
        Resource priority() const;

        /**
         * Set property 'priority'. The priority of a task. See http://en.wikipedia.org/wiki/Time_management 
         * for different approaches on finding priorities. 
         */
        void setPriority( const Resource& value );

        /**
         * \return The URI of the property 'priority'. 
         */
        static QUrl priorityUri();

        /**
         * Get property 'indexPosition'. The position of ordered subtasks 
         * regarding their parent task. It is possible to use this property 
         * because we allow for only one parent task. More exactly this 
         * should be realized by an rdf:List, but those has the disadvantage 
         * that no typing is possible. This issue needs to be clarified 
         * in future revisions. 
         */
        qint32 indexPosition() const;

        /**
         * Set property 'indexPosition'. The position of ordered subtasks 
         * regarding their parent task. It is possible to use this property 
         * because we allow for only one parent task. More exactly this 
         * should be realized by an rdf:List, but those has the disadvantage 
         * that no typing is possible. This issue needs to be clarified 
         * in future revisions. 
         */
        void setIndexPosition( const qint32& value );

        /**
         * \return The URI of the property 'indexPosition'. 
         */
        static QUrl indexPositionUri();

        /**
         * Get property 'targetEndTime'. 
         */
        QDateTime targetEndTime() const;

        /**
         * Set property 'targetEndTime'. 
         */
        void setTargetEndTime( const QDateTime& value );

        /**
         * \return The URI of the property 'targetEndTime'. 
         */
        static QUrl targetEndTimeUri();

        /**
         * Get property 'superTask'. 
         */
        Task superTask() const;

        /**
         * Set property 'superTask'. 
         */
        void setSuperTask( const Task& value );

        /**
         * \return The URI of the property 'superTask'. 
         */
        static QUrl superTaskUri();

        /**
         * Get property 'targetCompletion'. The degree (in percent) 
         * to which a task should be completed. This property has a rather 
         * rare usage, but one can imagine situations where one explicitly 
         * states that a task has only to be performed to a certain extend. 
         */
        double targetCompletion() const;

        /**
         * Set property 'targetCompletion'. The degree (in percent) 
         * to which a task should be completed. This property has a rather 
         * rare usage, but one can imagine situations where one explicitly 
         * states that a task has only to be performed to a certain extend. 
         */
        void setTargetCompletion( const double& value );

        /**
         * \return The URI of the property 'targetCompletion'. 
         */
        static QUrl targetCompletionUri();

        /**
         * Get property 'nextReviewIntervall'. The time when the next 
         * review should be performed 
         */
        qint64 nextReviewIntervall() const;

        /**
         * Set property 'nextReviewIntervall'. The time when the next 
         * review should be performed 
         */
        void setNextReviewIntervall( const qint64& value );

        /**
         * \return The URI of the property 'nextReviewIntervall'. 
         */
        static QUrl nextReviewIntervallUri();

        /**
         * Get property 'taskState'. The task state property allows tracking 
         * a task during its lifecycle. A tmo:taskState is required, initially 
         * it can e.g. be set to "new". 
         */
        Resource taskState() const;

        /**
         * Set property 'taskState'. The task state property allows tracking 
         * a task during its lifecycle. A tmo:taskState is required, initially 
         * it can e.g. be set to "new". 
         */
        void setTaskState( const Resource& value );

        /**
         * \return The URI of the property 'taskState'. 
         */
        static QUrl taskStateUri();

        /**
         * Get property 'delegability'. This property determines how 
         * appropriate a task is to be delegated to another Person. Some 
         * task have to be performed by the owning person and cannot be delegated. 
         */
        Resource delegability() const;

        /**
         * Set property 'delegability'. This property determines how 
         * appropriate a task is to be delegated to another Person. Some 
         * task have to be performed by the owning person and cannot be delegated. 
         */
        void setDelegability( const Resource& value );

        /**
         * \return The URI of the property 'delegability'. 
         */
        static QUrl delegabilityUri();

        /**
         * Get property 'taskName'. The tmo:taskName helps the user to 
         * identify a tmo:task in a list. It should be expressive enough 
         * to give a meaningful recognition. Details should be written 
         * in the tmo:taskDescription attribute instead. A tmo:taskName 
         * attribute is not allowed to contain line breaks. The name property 
         * is a sub-property of the nao:prefLabel , so this instance can 
         * by default be displayed correctly in places which are not specialized 
         * towards the TMO alone. 
         */
        QString taskName() const;

        /**
         * Set property 'taskName'. The tmo:taskName helps the user to 
         * identify a tmo:task in a list. It should be expressive enough 
         * to give a meaningful recognition. Details should be written 
         * in the tmo:taskDescription attribute instead. A tmo:taskName 
         * attribute is not allowed to contain line breaks. The name property 
         * is a sub-property of the nao:prefLabel , so this instance can 
         * by default be displayed correctly in places which are not specialized 
         * towards the TMO alone. 
         */
        void setTaskName( const QString& value );

        /**
         * \return The URI of the property 'taskName'. 
         */
        static QUrl taskNameUri();

        /**
         * Get property 'logEntry'. Tasks are not static entities which 
         * are created once and then they remain constant until they are 
         * finished. The tasks as we see it should be able to evolve together 
         * with the beliefs/ideas/thoughts of the human user. Tasks are 
         * not only requirements to be fulfilled but tasks can reflect 
         * the usage of task related information, e.g. attachment documents 
         * are added during the lifespan of tasks where the exact point 
         * of time of the attachment can be important to understand the 
         * task proceeding or if due dates might be shifted. This information 
         * can be kept. We leave it open how this is done in concrete cases. 
         */
        QList<Resource> logEntrys() const;

        /**
         * Set property 'logEntry'. Tasks are not static entities which 
         * are created once and then they remain constant until they are 
         * finished. The tasks as we see it should be able to evolve together 
         * with the beliefs/ideas/thoughts of the human user. Tasks are 
         * not only requirements to be fulfilled but tasks can reflect 
         * the usage of task related information, e.g. attachment documents 
         * are added during the lifespan of tasks where the exact point 
         * of time of the attachment can be important to understand the 
         * task proceeding or if due dates might be shifted. This information 
         * can be kept. We leave it open how this is done in concrete cases. 
         */
        void setLogEntrys( const QList<Resource>& value );

        /**
         * Add a value to property 'logEntry'. Tasks are not static entities 
         * which are created once and then they remain constant until they 
         * are finished. The tasks as we see it should be able to evolve together 
         * with the beliefs/ideas/thoughts of the human user. Tasks are 
         * not only requirements to be fulfilled but tasks can reflect 
         * the usage of task related information, e.g. attachment documents 
         * are added during the lifespan of tasks where the exact point 
         * of time of the attachment can be important to understand the 
         * task proceeding or if due dates might be shifted. This information 
         * can be kept. We leave it open how this is done in concrete cases. 
         */
        void addLogEntry( const Resource& value );

        /**
         * \return The URI of the property 'logEntry'. 
         */
        static QUrl logEntryUri();

        /**
         * Get property 'actualEndTime'. 
         */
        QDateTime actualEndTime() const;

        /**
         * Set property 'actualEndTime'. 
         */
        void setActualEndTime( const QDateTime& value );

        /**
         * \return The URI of the property 'actualEndTime'. 
         */
        static QUrl actualEndTimeUri();

        /**
         * Get property 'contextThread'. This property realizes the 
         * connection to the user work context by an relation to an workcontext:contextThread. 
         */
        Resource contextThread() const;

        /**
         * Set property 'contextThread'. This property realizes the 
         * connection to the user work context by an relation to an workcontext:contextThread. 
         */
        void setContextThread( const Resource& value );

        /**
         * \return The URI of the property 'contextThread'. 
         */
        static QUrl contextThreadUri();

        /**
         * Get property 'attachment'. Connects the complex (n-ary, role 
         * described) Attachment and the task. See tmo:Attachment and 
         * section 3 for details. 
         */
        QList<Resource> attachments() const;

        /**
         * Set property 'attachment'. Connects the complex (n-ary, role 
         * described) Attachment and the task. See tmo:Attachment and 
         * section 3 for details. 
         */
        void setAttachments( const QList<Resource>& value );

        /**
         * Add a value to property 'attachment'. Connects the complex 
         * (n-ary, role described) Attachment and the task. See tmo:Attachment 
         * and section 3 for details. 
         */
        void addAttachment( const Resource& value );

        /**
         * \return The URI of the property 'attachment'. 
         */
        static QUrl attachmentUri();

        /**
         * Get property 'actualTimeUsage'. Represents the amount of 
         * time actually spend on a task. 
         */
        qint32 actualTimeUsage() const;

        /**
         * Set property 'actualTimeUsage'. Represents the amount of 
         * time actually spend on a task. 
         */
        void setActualTimeUsage( const qint32& value );

        /**
         * \return The URI of the property 'actualTimeUsage'. 
         */
        static QUrl actualTimeUsageUri();

        /**
         * Get property 'dependency'. Tasks can be associated with each 
         * other in various ways. Those associations are realized by instances 
         * of tmo:TaskDependency. 
         */
        QList<Resource> dependencys() const;

        /**
         * Set property 'dependency'. Tasks can be associated with each 
         * other in various ways. Those associations are realized by instances 
         * of tmo:TaskDependency. 
         */
        void setDependencys( const QList<Resource>& value );

        /**
         * Add a value to property 'dependency'. Tasks can be associated 
         * with each other in various ways. Those associations are realized 
         * by instances of tmo:TaskDependency. 
         */
        void addDependency( const Resource& value );

        /**
         * \return The URI of the property 'dependency'. 
         */
        static QUrl dependencyUri();

        /**
         * Get property 'targetStartTime'. 
         */
        QDateTime targetStartTime() const;

        /**
         * Set property 'targetStartTime'. 
         */
        void setTargetStartTime( const QDateTime& value );

        /**
         * \return The URI of the property 'targetStartTime'. 
         */
        static QUrl targetStartTimeUri();

        /**
         * Get property 'taskId'. The tmo:taskId allows a unique identification 
         * of a task object within the range of all Nepomuk objects. The 
         * tmo:taskId is automatically generated during the creation 
         * of a task. Using UUID is an valid approach to generate id's. 
         */
        QString taskId() const;

        /**
         * Set property 'taskId'. The tmo:taskId allows a unique identification 
         * of a task object within the range of all Nepomuk objects. The 
         * tmo:taskId is automatically generated during the creation 
         * of a task. Using UUID is an valid approach to generate id's. 
         */
        void setTaskId( const QString& value );

        /**
         * \return The URI of the property 'taskId'. 
         */
        static QUrl taskIdUri();

        /**
         * Get property 'taskEffort'. 
         */
        QList<Resource> taskEfforts() const;

        /**
         * Set property 'taskEffort'. 
         */
        void setTaskEfforts( const QList<Resource>& value );

        /**
         * Add a value to property 'taskEffort'. 
         */
        void addTaskEffort( const Resource& value );

        /**
         * \return The URI of the property 'taskEffort'. 
         */
        static QUrl taskEffortUri();

        /**
         * Get property 'abilityCarrierInvolvement'. Connects the 
         * complex (n-ary, role described) AbilityCarrierInvolvement 
         * and the task. See tmo:AbilityCarrierInvolvement and section 
         * 3 for details. 
         */
        QList<Resource> abilityCarrierInvolvements() const;

        /**
         * Set property 'abilityCarrierInvolvement'. Connects the 
         * complex (n-ary, role described) AbilityCarrierInvolvement 
         * and the task. See tmo:AbilityCarrierInvolvement and section 
         * 3 for details. 
         */
        void setAbilityCarrierInvolvements( const QList<Resource>& value );

        /**
         * Add a value to property 'abilityCarrierInvolvement'. Connects 
         * the complex (n-ary, role described) AbilityCarrierInvolvement 
         * and the task. See tmo:AbilityCarrierInvolvement and section 
         * 3 for details. 
         */
        void addAbilityCarrierInvolvement( const Resource& value );

        /**
         * \return The URI of the property 'abilityCarrierInvolvement'. 
         */
        static QUrl abilityCarrierInvolvementUri();

        /**
         * Get property 'actualStartTime'. 
         */
        QDateTime actualStartTime() const;

        /**
         * Set property 'actualStartTime'. 
         */
        void setActualStartTime( const QDateTime& value );

        /**
         * \return The URI of the property 'actualStartTime'. 
         */
        static QUrl actualStartTimeUri();

        /**
         * Get property 'taskTransmission'. Creates the connection 
         * to the tmo:TaskTransmission where the details of a transmission 
         * act are aggregated. 
         */
        QList<Resource> taskTransmissions() const;

        /**
         * Set property 'taskTransmission'. Creates the connection 
         * to the tmo:TaskTransmission where the details of a transmission 
         * act are aggregated. 
         */
        void setTaskTransmissions( const QList<Resource>& value );

        /**
         * Add a value to property 'taskTransmission'. Creates the connection 
         * to the tmo:TaskTransmission where the details of a transmission 
         * act are aggregated. 
         */
        void addTaskTransmission( const Resource& value );

        /**
         * \return The URI of the property 'taskTransmission'. 
         */
        static QUrl taskTransmissionUri();

        /**
         * Get property 'taskPrivacyState'. For the separation between 
         * professional and private purpose of a task, this attribute 
         * provides with the values "professional/private" a high level 
         * separation of privacy in terms of setting distribution rights 
         * to other users for the task. This separation may arise as a general 
         * Nepomuk issue and may therefore be handled in conjunction with 
         * a privacy preserving SSD architecture. 
         */
        Resource taskPrivacyState() const;

        /**
         * Set property 'taskPrivacyState'. For the separation between 
         * professional and private purpose of a task, this attribute 
         * provides with the values "professional/private" a high level 
         * separation of privacy in terms of setting distribution rights 
         * to other users for the task. This separation may arise as a general 
         * Nepomuk issue and may therefore be handled in conjunction with 
         * a privacy preserving SSD architecture. 
         */
        void setTaskPrivacyState( const Resource& value );

        /**
         * \return The URI of the property 'taskPrivacyState'. 
         */
        static QUrl taskPrivacyStateUri();

        /**
         * Get property 'lastReviewDate'. The last date a task was reviewed. 
         */
        QDateTime lastReviewDate() const;

        /**
         * Set property 'lastReviewDate'. The last date a task was reviewed. 
         */
        void setLastReviewDate( const QDateTime& value );

        /**
         * \return The URI of the property 'lastReviewDate'. 
         */
        static QUrl lastReviewDateUri();

        /**
         * Get property 'taskGoal'. The goal which is to be accomplished 
         * by a task. Goal orientation is a primary objective in task management. 
         * For goal articulation there are different possibilities on 
         * a tmo:Task instance. First there is a set of already articulated 
         * goals, which are made available to the user for simple selection. 
         * In contrast to Web 2.0 style applications those goals can be 
         * conceptualized as instances. The semantic desktop user is 
         * free to enhance his present goal repertoire by new instances. 
         * In principle the user can also use the tmo:taskName property 
         * for short sequences of text or the tmo:taskDescription property 
         * with its ability to display longer sequences of text to manifest 
         * goals. In the long term this is not desirable. When a phrase is 
         * repeatedly used as a goal, it should be uses as a typed "marker" 
         * and a instance should be used. The range is kept very flexible 
         * as a Thing. 
         */
        QList<Resource> taskGoals() const;

        /**
         * Set property 'taskGoal'. The goal which is to be accomplished 
         * by a task. Goal orientation is a primary objective in task management. 
         * For goal articulation there are different possibilities on 
         * a tmo:Task instance. First there is a set of already articulated 
         * goals, which are made available to the user for simple selection. 
         * In contrast to Web 2.0 style applications those goals can be 
         * conceptualized as instances. The semantic desktop user is 
         * free to enhance his present goal repertoire by new instances. 
         * In principle the user can also use the tmo:taskName property 
         * for short sequences of text or the tmo:taskDescription property 
         * with its ability to display longer sequences of text to manifest 
         * goals. In the long term this is not desirable. When a phrase is 
         * repeatedly used as a goal, it should be uses as a typed "marker" 
         * and a instance should be used. The range is kept very flexible 
         * as a Thing. 
         */
        void setTaskGoals( const QList<Resource>& value );

        /**
         * Add a value to property 'taskGoal'. The goal which is to be accomplished 
         * by a task. Goal orientation is a primary objective in task management. 
         * For goal articulation there are different possibilities on 
         * a tmo:Task instance. First there is a set of already articulated 
         * goals, which are made available to the user for simple selection. 
         * In contrast to Web 2.0 style applications those goals can be 
         * conceptualized as instances. The semantic desktop user is 
         * free to enhance his present goal repertoire by new instances. 
         * In principle the user can also use the tmo:taskName property 
         * for short sequences of text or the tmo:taskDescription property 
         * with its ability to display longer sequences of text to manifest 
         * goals. In the long term this is not desirable. When a phrase is 
         * repeatedly used as a goal, it should be uses as a typed "marker" 
         * and a instance should be used. The range is kept very flexible 
         * as a Thing. 
         */
        void addTaskGoal( const Resource& value );

        /**
         * \return The URI of the property 'taskGoal'. 
         */
        static QUrl taskGoalUri();

        /**
         * Get property 'subTask'. Tasks can be hierarchically arranged. 
         * See tmo:Task for details. 
         */
        QList<Task> subTasks() const;

        /**
         * Set property 'subTask'. Tasks can be hierarchically arranged. 
         * See tmo:Task for details. 
         */
        void setSubTasks( const QList<Task>& value );

        /**
         * Add a value to property 'subTask'. Tasks can be hierarchically 
         * arranged. See tmo:Task for details. 
         */
        void addSubTask( const Task& value );

        /**
         * \return The URI of the property 'subTask'. 
         */
        static QUrl subTaskUri();

        /**
         * Get property 'taskSource'. The taskSopurce property can be 
         * used to state from which sources a task was derived, e.g from 
         * another task or from an task pattern 
         */
        Resource taskSource() const;

        /**
         * Set property 'taskSource'. The taskSopurce property can be 
         * used to state from which sources a task was derived, e.g from 
         * another task or from an task pattern 
         */
        void setTaskSource( const Resource& value );

        /**
         * \return The URI of the property 'taskSource'. 
         */
        static QUrl taskSourceUri();

        /**
         * Get property 'subTaskOrdering'. This property allows to optionally 
         * state the ordering of the subtasks listed in the tmo:subTask 
         * property a task. This is only for ordering/sorting in GUIs, 
         * the semantic relation is defined in tmo:subTask, and if this 
         * and tmo:subTask differ, tmo:subTasks is the correct list (while 
         * ordering is lost). See also tmo:indexPosition for a property 
         * which realizes this by means of an workaround since the support 
         * of rdf:List is often limited on tool support. Also there is no 
         * support for range restrictions. 
         */
        Resource subTaskOrdering() const;

        /**
         * Set property 'subTaskOrdering'. This property allows to optionally 
         * state the ordering of the subtasks listed in the tmo:subTask 
         * property a task. This is only for ordering/sorting in GUIs, 
         * the semantic relation is defined in tmo:subTask, and if this 
         * and tmo:subTask differ, tmo:subTasks is the correct list (while 
         * ordering is lost). See also tmo:indexPosition for a property 
         * which realizes this by means of an workaround since the support 
         * of rdf:List is often limited on tool support. Also there is no 
         * support for range restrictions. 
         */
        void setSubTaskOrdering( const Resource& value );

        /**
         * \return The URI of the property 'subTaskOrdering'. 
         */
        static QUrl subTaskOrderingUri();

        /**
         * Get property 'importance'. The importance of a task according 
         * to the "ABC analysis" (see http://en.wikipedia.org/wiki/Time_management). 
         */
        Resource importance() const;

        /**
         * Set property 'importance'. The importance of a task according 
         * to the "ABC analysis" (see http://en.wikipedia.org/wiki/Time_management). 
         */
        void setImportance( const Resource& value );

        /**
         * \return The URI of the property 'importance'. 
         */
        static QUrl importanceUri();

        /**
         * Get property 'targetTimeUsage'. Represents the amount of 
         * time planned to be spend on a task. 
         */
        qint32 targetTimeUsage() const;

        /**
         * Set property 'targetTimeUsage'. Represents the amount of 
         * time planned to be spend on a task. 
         */
        void setTargetTimeUsage( const qint32& value );

        /**
         * \return The URI of the property 'targetTimeUsage'. 
         */
        static QUrl targetTimeUsageUri();

        /**
         * Get all resources that have this resource set as property 'containsTask'. 
         * \sa ResourceManager::allResourcesWithProperty 
         */
        QList<Resource> containsTaskOf() const;

        /**
         * Get all resources that have this resource set as property 'dependencyMemberA'. 
         * The semantic of this relation is defined in the subclass of undirected 
         * Dependency on which this property is stated. (The subject of 
         * the statement where this property is expressed). Used in the 
         * role based modeling of tmo:TaskDependency. See section 3. 
         * \sa ResourceManager::allResourcesWithProperty 
         */
        QList<Resource> dependencyMemberAOf() const;

        /**
         * Get all resources that have this resource set as property 'dependencyMemberB'. 
         * The semantic of this relation is defined in the subclass of undirected 
         * Dependency on which this property is stated. (The subject of 
         * the statement where this property is expressed). Used in the 
         * role based modeling of tmo:TaskDependency. See section 3. 
         * \sa ResourceManager::allResourcesWithProperty 
         */
        QList<Resource> dependencyMemberBOf() const;

        /**
         * Get all resources that have this resource set as property 'taskReference'. 
         * Abstract super-property for various properties which establish 
         * a connections to a task. \sa ResourceManager::allResourcesWithProperty 
         */
        QList<Resource> taskReferenceOf() const;

        /**
         * Retrieve a list of all available Task resources. This list consists 
         * of all resource of type Task that are stored in the local Nepomuk 
         * meta data storage and any changes made locally. Be aware that 
         * in some cases this list can get very big. Then it might be better 
         * to use libKNep directly. 
         */
        static QList<Task> allTasks();


        /**
         * \return The URI of the resource type that is used in Task instances.
         */
        static QString resourceTypeUri();

    protected:
       Task( const QString& uri, const QUrl& type );
       Task( const QUrl& uri, const QUrl& type );
       Task( const QString& uri, const QUrl& type, ResourceManager* );
       Task( const QUrl& uri, const QUrl& type, ResourceManager* );
    };
}

#endif
