/*
   Copyright (c) 2009-2010 Sebastian Trueg <trueg@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mainwindow.h"

#include "taskmodel.h"
#include "taskfiltermodel.h"
#include "daterangeselectionwidget.h"
#include "daterange.h"
#include "task.h"
#include "resourceactions.h"

#include <QtGui/QHBoxLayout>
#include <QtGui/QTreeView>
#include <QtGui/QKeyEvent>
#include <QtGui/QContextMenuEvent>

#include <KLineEdit>
#include <KMenu>

#include <Nepomuk/Resource>
#include <nepomuk/tagwidget.h>


MainWindow::MainWindow()
    : QMainWindow()
{
    // setup UI
    setWindowTitle( i18n( "Nepomuk - Task Manager" ) );
    QWidget* w = new QWidget( this );
    setupUi( w );
    setCentralWidget( w );

    m_taskFilterModel = new Nepomuk::TaskFilterModel(this);
    m_taskModel = new Nepomuk::TaskModel(this);
    m_taskFilterModel->setSourceModel( m_taskModel );
    m_taskView->setModel( m_taskFilterModel );

    m_resourceActions = new Nepomuk::ResourceActions( this );

    connect( m_checkShowCompletedTasks, SIGNAL( toggled( bool ) ),
             m_taskFilterModel, SLOT( setShowCompletedTasks( bool ) ) );
    connect( m_dateRangeSelectionWidget, SIGNAL( rangeChanged( DateRange ) ),
             m_taskFilterModel, SLOT( setFilterDateRange( DateRange ) ) );
    connect( m_taskView->selectionModel(), SIGNAL( selectionChanged( QItemSelection, QItemSelection ) ),
             this, SLOT( slotSelectionChanged( QItemSelection, QItemSelection ) ) );

    m_editNewTask->installEventFilter( this );
    m_taskView->installEventFilter( this );
    m_editNewTask->hide();
    m_tagWidget->setEnabled( false );
}


MainWindow::~MainWindow()
{
}


void MainWindow::slotSelectionChanged( const QItemSelection& selected, const QItemSelection& deselected )
{
    if ( selected.indexes().isEmpty() ) {
        m_tagWidget->setTaggedResource( Nepomuk::Resource() );
        m_tagWidget->setEnabled( false );
    }
    else {
        Nepomuk::Resource res = m_taskModel->resourceForIndex( m_taskFilterModel->mapToSource( selected.indexes().first() ) );
        m_tagWidget->setTaggedResource( res );
        m_tagWidget->setEnabled( true );
        m_resourceActions->setResource( res );
    }
}


void MainWindow::keyPressEvent( QKeyEvent* event )
{
    if ( event->key() == Qt::Key_Return ) {
        m_editNewTask->show();
        m_editNewTask->setFocus();
    }
}


bool MainWindow::eventFilter( QObject* o, QEvent* e )
{
    if ( o == m_editNewTask ) {
        if ( e->type() == QEvent::KeyPress ) {
            QKeyEvent* ke = static_cast<QKeyEvent*>( e );
            if ( ke->key() == Qt::Key_Escape ) {
                m_editNewTask->clear();
                m_editNewTask->hide();
                return true;
            }
            else if ( ke->key() == Qt::Key_Return ||
                      ke->key() == Qt::Key_Enter ) {
                // create a new task
                // FIXME: use Akonadi
                Nepomuk::Task newTask;
                newTask.setLabel( m_editNewTask->text() );
                m_editNewTask->clear();
                m_editNewTask->hide();
                return true;
            }
        }
    }
    else if ( o == m_taskView ) {
        if ( e->type() == QEvent::ContextMenu ) {
            QContextMenuEvent* cme = static_cast<QContextMenuEvent*>( e );
            KMenu popup;
            m_resourceActions->populateMenu( &popup );
            popup.exec( cme->globalPos() );
            return true;
        }
    }

    return false;
}

#include "mainwindow.moc"
