/*
   Copyright (C) 2009 by Charles Wautier <charles.wautier@gmail.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "tasktreedialog.h"
#include "tmo.h"

#include <kresourcetreeview.h>
#include <QWidget>
#include <QHeaderView>
#include <QAbstractItemView>
#include <klocale.h>
#include <Nepomuk/Resource>
#include "pimo.h"
#include <kdebug.h>
#include <QLabel>

class TaskTreeDialog::Private
{
public :
    KResourceTreeView* ftv;
    Private(QWidget* parent);
    ~Private();
};

TaskTreeDialog::Private::Private(QWidget* parent)
        : ftv(new KResourceTreeView(parent))
{
    ftv->setRootUrl(Nepomuk::Vocabulary::TMO::Task(), Nepomuk::Vocabulary::TMO::superTask(), Nepomuk::Vocabulary::PIMO::isRelated());
}

TaskTreeDialog::Private::~Private()
{
    delete ftv;
}

TaskTreeDialog::TaskTreeDialog(QWidget* parent)
        : TaskTreeDialog::KDialog(parent), d(new Private(this))
{
    setMainWidget(d->ftv);
    resize(400, 300);
}

TaskTreeDialog::~TaskTreeDialog()
{
    delete d;
}

KResourceTreeView* TaskTreeDialog::kResourceTreeView()
{
    return d->ftv;
}

void TaskTreeDialog::slotTaskSelected(const KUrl &url)
{
    enableButtonOk(true);
}

Nepomuk::Resource TaskTreeDialog::selectOneTask(QWidget* parent)
{
    TaskTreeDialog dlg(parent);
    dlg.setCaption(i18n("Task selection"));
    dlg.enableButtonOk(false);
    dlg.d->ftv->setDirOnlyMode(true);
    dlg.d->ftv->setSelectionMode(QAbstractItemView::SingleSelection);

    connect(dlg.d->ftv, SIGNAL(currentChanged(const KUrl &)),
            &dlg, SLOT(slotTaskSelected(const KUrl &)));

    if (dlg.exec() == KDialog::Accepted && !dlg.d->ftv->selectedUri().isEmpty()) {
        Nepomuk::Resource res = Nepomuk::Resource(dlg.d->ftv->selectedUri(), Nepomuk::Vocabulary::TMO::Task())  ;
        return res;
    } else {
        return Nepomuk::Resource();
    }
}
